@@ -193,7 +193,7 @@ int classBrowse::RowAction( const QString &shortDesc, const QString &sql, const
       my_msgBox( "classBrowse", "SQLExecute", nReturn, NULL, NULL, stmt(), sql ) ;
 
     // GET NUMBER OF ROWS AFFECTED
-    SQLINTEGER nRows = 0 ;
+    SQLLEN nRows = 0 ;
     if (!SQL_SUCCEEDED(SQLRowCount( stmt(), &nRows ) ) )
       nRows = 0 ;
 
