@@ -27,9 +27,22 @@
  *
  **********************************************************************
  *
- * $Id: SQLAllocHandle.c,v 1.4 2002/12/12 18:21:21 lurcher Exp $
+ * $Id: SQLAllocHandle.c,v 1.6 2003/10/30 18:20:45 lurcher Exp $
  *
  * $Log: SQLAllocHandle.c,v $
+ * Revision 1.6  2003/10/30 18:20:45  lurcher
+ *
+ * Fix broken thread protection
+ * Remove SQLNumResultCols after execute, lease S4/S% to driver
+ * Fix string overrun in SQLDriverConnect
+ * Add initial support for Interix
+ *
+ * Revision 1.5  2003/02/25 13:28:21  lurcher
+ *
+ * Allow errors on the drivers AllocHandle to be reported
+ * Fix a problem that caused errors to not be reported in the log
+ * Remove a redundant line from the spec file
+ *
  * Revision 1.4  2002/12/12 18:21:21  lurcher
  *
  * Fix bug where if a SQLAllocHandle in the driver failed, the driver manager
@@ -217,7 +230,7 @@
 #include <uodbc_stats.h>
 #endif
 
-static char const rcsid[]= "$RCSfile: SQLAllocHandle.c,v $ $Revision: 1.4 $";
+static char const rcsid[]= "$RCSfile: SQLAllocHandle.c,v $ $Revision: 1.6 $";
 
 /*
  * connection pooling stuff
@@ -347,9 +360,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                         ERROR_HY009, NULL, 
                         SQL_OV_ODBC3 );
 
-                thread_release( SQL_HANDLE_ENV, environment );
-
-                return function_return( environment, SQL_ERROR );
+                return function_return( SQL_HANDLE_ENV, environment, SQL_ERROR );
             }
 
             /*
@@ -368,9 +379,9 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                         ERROR_HY010, NULL,
                         SQL_OV_ODBC3 );
 
-                thread_release( SQL_HANDLE_ENV, environment );
+                *output_handle = SQL_NULL_HDBC;
 
-                return function_return( environment, SQL_ERROR );
+                return function_return( SQL_HANDLE_ENV, environment, SQL_ERROR );
             }
 
             connection = __alloc_dbc();
@@ -386,9 +397,9 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                     ERROR_HY013, NULL,
                     environment -> requested_version );
 
-                thread_release( SQL_HANDLE_ENV, environment );
+                *output_handle = SQL_NULL_HDBC;
 
-                return function_return( environment, SQL_ERROR );
+                return function_return( SQL_HANDLE_ENV, environment, SQL_ERROR );
             }
 
             /*
@@ -514,9 +525,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                         ERROR_HY009, NULL, 
                         connection -> environment -> requested_version  );
 
-                thread_release( SQL_HANDLE_DBC, connection );
-
-                return function_return( connection , SQL_ERROR );
+                return function_return( SQL_HANDLE_DBC, connection , SQL_ERROR );
             }
 
             if ( connection -> state == STATE_C1 ||
@@ -533,9 +542,9 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                         ERROR_08003, NULL,
                         connection -> environment -> requested_version );
 
-                thread_release( SQL_HANDLE_DBC, connection );
+                *output_handle = SQL_NULL_HSTMT;
 
-                return function_return( connection, SQL_ERROR );
+                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
             }
 
             statement = __alloc_stmt();
@@ -551,9 +560,9 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                         ERROR_HY013, NULL,
                         connection -> environment -> requested_version );
 
-                thread_release( SQL_HANDLE_DBC, connection );
+                *output_handle = SQL_NULL_HSTMT;
 
-                return function_return( connection, SQL_ERROR );
+                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
             }
 
             /*
@@ -597,9 +606,9 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
                     __release_stmt( statement );
 
-                    thread_release( SQL_HANDLE_DBC, connection );
+                    *output_handle = SQL_NULL_HSTMT;
 
-                    return function_return( connection, SQL_ERROR );
+                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
                 }
             }
             else
@@ -639,9 +648,9 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
                     __release_stmt( statement );
 
-                    thread_release( SQL_HANDLE_DBC, connection );
-
-                    return function_return( connection, SQL_ERROR );
+                    *output_handle = SQL_NULL_HSTMT;
+                
+                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
                 }
             }
 
@@ -706,9 +715,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
                             __release_stmt( statement );
 
-                            thread_release( SQL_HANDLE_DBC, connection );
-
-                            return function_return( connection, SQL_ERROR );
+                            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
                         }
                         statement -> implicit_ard = statement -> ard;
                         statement -> ard -> implicit = 1;
@@ -749,9 +756,9 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
         
                             __release_stmt( statement );
 
-                            thread_release( SQL_HANDLE_DBC, connection );
+                            *output_handle = SQL_NULL_HSTMT;
 
-                            return function_return( connection, SQL_ERROR );
+                            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
                         }
                         statement -> implicit_apd = statement -> apd;
                         statement -> apd -> implicit = 1;
@@ -792,9 +799,9 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
                             __release_stmt( statement );
 
-                            thread_release( SQL_HANDLE_DBC, connection );
+                            *output_handle = SQL_NULL_HSTMT;
 
-                            return function_return( connection, SQL_ERROR );
+                            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
                         }
                         statement -> implicit_ird = statement -> ird;
                         statement -> ird -> implicit = 1;
@@ -833,9 +840,9 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                                     ERROR_HY013, NULL,
                                     connection -> environment -> requested_version );
 
-                            thread_release( SQL_HANDLE_DBC, connection );
+                            *output_handle = SQL_NULL_HSTMT;
 
-                            return function_return( connection, SQL_ERROR );
+                            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
                         }
                         statement -> implicit_ipd = statement -> ipd;
                         statement -> ipd -> implicit = 1;
@@ -872,9 +879,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                                UODBC_STATS_TYPE_HSTMT, (void *)1);
 #endif
 
-            thread_release( SQL_HANDLE_DBC, connection );
-
-            return function_return( connection, ret );
+            return function_return( SQL_HANDLE_DBC, connection, ret );
         }
         break;
 
@@ -928,9 +933,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                     ERROR_HY009, NULL, 
                     connection -> environment -> requested_version  );
 
-            thread_release( SQL_HANDLE_DBC, connection );
-
-            return function_return( connection , SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection , SQL_ERROR );
         }
 
         if ( connection -> state == STATE_C1 ||
@@ -947,9 +950,9 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                     ERROR_08003, NULL,
                     connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_DBC, connection );
+            *output_handle = SQL_NULL_HDESC;
 
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
 
         descriptor = __alloc_desc();
@@ -965,9 +968,9 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                     ERROR_HY013, NULL,
                     connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_DBC, connection );
+            *output_handle = SQL_NULL_HDESC;
 
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
 
         /*
@@ -999,9 +1002,9 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
             __release_desc( descriptor );
 
-            thread_release( SQL_HANDLE_DBC, connection );
+            *output_handle = SQL_NULL_HDESC;
 
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
 
         if ( SQL_SUCCEEDED( ret ))
@@ -1037,9 +1040,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                            (void *)1);
 #endif
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, ret );
+        return function_return( SQL_HANDLE_DBC, connection, ret );
         }
         break;
 
