@@ -27,9 +27,16 @@
  *
  **********************************************************************
  *
- * $Id: SQLBindCol.c,v 1.3 2002/12/05 17:44:30 lurcher Exp $
+ * $Id: SQLBindCol.c,v 1.4 2003/10/30 18:20:45 lurcher Exp $
  *
  * $Log: SQLBindCol.c,v $
+ * Revision 1.4  2003/10/30 18:20:45  lurcher
+ *
+ * Fix broken thread protection
+ * Remove SQLNumResultCols after execute, lease S4/S% to driver
+ * Fix string overrun in SQLDriverConnect
+ * Add initial support for Interix
+ *
  * Revision 1.3  2002/12/05 17:44:30  lurcher
  *
  * Display unknown return values in return logging
@@ -108,7 +115,7 @@
 
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLBindCol.c,v $ $Revision: 1.3 $";
+static char const rcsid[]= "$RCSfile: SQLBindCol.c,v $ $Revision: 1.4 $";
 
 SQLRETURN SQLBindCol( SQLHSTMT statement_handle,
 		   SQLUSMALLINT column_number,
@@ -175,9 +182,7 @@ SQLRETURN SQLBindCol( SQLHSTMT statement_handle,
                 ERROR_HY090, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_STMT, statement );
-
-        return function_return( statement, SQL_ERROR );
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     /*
@@ -208,9 +213,7 @@ SQLRETURN SQLBindCol( SQLHSTMT statement_handle,
                 ERROR_HY010, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_STMT, statement );
-
-        return function_return( statement, SQL_ERROR );
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     if ( !CHECK_SQLBINDCOL( statement -> connection ))
@@ -225,9 +228,7 @@ SQLRETURN SQLBindCol( SQLHSTMT statement_handle,
                 ERROR_IM001, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_STMT, statement );
-
-        return function_return( statement, SQL_ERROR );
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     ret = SQLBINDCOL( statement -> connection ,
@@ -253,7 +254,5 @@ SQLRETURN SQLBindCol( SQLHSTMT statement_handle,
                 statement -> msg );
     }
 
-    thread_release( SQL_HANDLE_STMT, statement );
-
-    return function_return( statement, ret );
+    return function_return( SQL_HANDLE_STMT, statement, ret );
 }
