@@ -27,9 +27,16 @@
  *
  **********************************************************************
  *
- * $Id: SQLBindParam.c,v 1.4 2002/12/05 17:44:30 lurcher Exp $
+ * $Id: SQLBindParam.c,v 1.5 2003/10/30 18:20:45 lurcher Exp $
  *
  * $Log: SQLBindParam.c,v $
+ * Revision 1.5  2003/10/30 18:20:45  lurcher
+ *
+ * Fix broken thread protection
+ * Remove SQLNumResultCols after execute, lease S4/S% to driver
+ * Fix string overrun in SQLDriverConnect
+ * Add initial support for Interix
+ *
  * Revision 1.4  2002/12/05 17:44:30  lurcher
  *
  * Display unknown return values in return logging
@@ -112,7 +119,7 @@
 
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLBindParam.c,v $ $Revision: 1.4 $";
+static char const rcsid[]= "$RCSfile: SQLBindParam.c,v $ $Revision: 1.5 $";
 
 SQLRETURN SQLBindParam( SQLHSTMT statement_handle,
            SQLUSMALLINT parameter_number,
@@ -187,9 +194,7 @@ SQLRETURN SQLBindParam( SQLHSTMT statement_handle,
                 statement -> connection -> environment -> requested_version,
                 SQL_API_SQLBINDPARAM );
 
-        thread_release( SQL_HANDLE_STMT, statement );
-
-        return function_return( statement, SQL_ERROR );
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     if ( parameter_value == NULL &&
@@ -205,9 +210,7 @@ SQLRETURN SQLBindParam( SQLHSTMT statement_handle,
                 ERROR_HY009, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_STMT, statement );
-
-        return function_return( statement, SQL_ERROR );
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     /*
@@ -230,9 +233,7 @@ SQLRETURN SQLBindParam( SQLHSTMT statement_handle,
                 ERROR_HY010, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_STMT, statement );
-
-        return SQL_ERROR;
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     if ( CHECK_SQLBINDPARAM( statement -> connection ))
@@ -265,9 +266,7 @@ SQLRETURN SQLBindParam( SQLHSTMT statement_handle,
                     ERROR_IM001, NULL,
                     statement -> connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_STMT, statement );
-
-            return function_return( statement, SQL_ERROR );
+            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
 
         /*
@@ -303,7 +302,5 @@ SQLRETURN SQLBindParam( SQLHSTMT statement_handle,
                 statement -> msg );
     }
 
-    thread_release( SQL_HANDLE_STMT, statement );
-
-    return function_return( statement, ret );
+    return function_return( SQL_HANDLE_STMT, statement, ret );
 }
