@@ -27,9 +27,38 @@
  *
  **********************************************************************
  *
- * $Id: SQLColAttribute.c,v 1.7 2002/12/05 17:44:30 lurcher Exp $
+ * $Id: SQLColAttribute.c,v 1.14 2004/11/22 17:02:48 lurcher Exp $
  *
  * $Log: SQLColAttribute.c,v $
+ * Revision 1.14  2004/11/22 17:02:48  lurcher
+ * Fix unicode/ansi conversion in the SQLGet functions
+ *
+ * Revision 1.13  2004/10/30 20:19:21  peteralexharvey
+ * ODBC spec says last arg for SQLColAttribute() is SQLPOINTER not (SQLEN*).
+ * So switched back to SQLPOINTER.
+ *
+ * Revision 1.12  2004/10/29 10:00:35  lurcher
+ * Fix SQLColAttribute protype
+ *
+ * Revision 1.11  2003/10/30 18:20:45  lurcher
+ *
+ * Fix broken thread protection
+ * Remove SQLNumResultCols after execute, lease S4/S% to driver
+ * Fix string overrun in SQLDriverConnect
+ * Add initial support for Interix
+ *
+ * Revision 1.10  2003/04/10 13:45:51  lurcher
+ *
+ * Alter the way that SQLDataSources returns the description field (again)
+ *
+ * Revision 1.9  2003/04/09 08:42:18  lurcher
+ *
+ * Allow setting of odbcinstQ lib from odbcinst.ini and Environment
+ *
+ * Revision 1.8  2003/02/27 12:19:39  lurcher
+ *
+ * Add the A functions as well as the W
+ *
  * Revision 1.7  2002/12/05 17:44:30  lurcher
  *
  * Display unknown return values in return logging
@@ -140,7 +169,7 @@
 
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLColAttribute.c,v $ $Revision: 1.7 $";
+static char const rcsid[]= "$RCSfile: SQLColAttribute.c,v $ $Revision: 1.14 $";
 
 SQLINTEGER map_ca_odbc3_to_2( SQLINTEGER field_identifier )
 {
@@ -181,6 +210,23 @@ SQLINTEGER map_ca_odbc3_to_2( SQLINTEGER field_identifier )
     return field_identifier;
 }
 
+SQLRETURN SQLColAttributeA( SQLHSTMT statement_handle,
+           SQLSMALLINT column_number,
+           SQLSMALLINT field_identifier,
+           SQLPOINTER character_attribute,
+           SQLSMALLINT buffer_length,
+           SQLSMALLINT *string_length,
+           SQLPOINTER numeric_attribute )
+{
+    return SQLColAttribute( statement_handle,
+                            (SQLUSMALLINT) column_number,
+                            (SQLUSMALLINT) field_identifier,
+                            character_attribute,
+                            buffer_length,
+                            string_length,
+                            numeric_attribute );
+}
+
 SQLRETURN SQLColAttribute ( SQLHSTMT statement_handle,
            SQLUSMALLINT column_number,
            SQLUSMALLINT field_identifier,
@@ -234,6 +280,8 @@ SQLRETURN SQLColAttribute ( SQLHSTMT statement_handle,
                 statement -> msg );
     }
 
+    thread_protect( SQL_HANDLE_STMT, statement );
+
     if ( column_number == 0 &&
             statement -> bookmarks_on == SQL_UB_OFF && statement -> connection -> bookmarks_on == SQL_UB_OFF &&
             field_identifier != SQL_DESC_COUNT )
@@ -248,7 +296,8 @@ SQLRETURN SQLColAttribute ( SQLHSTMT statement_handle,
                 ERROR_07009, NULL,
                 statement -> connection -> environment -> requested_version,
                 SQL_API_SQLCOLATTRIBUTE );
-        return function_return( statement, SQL_ERROR );
+
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
 	/*
@@ -281,7 +330,8 @@ SQLRETURN SQLColAttribute ( SQLHSTMT statement_handle,
         __post_internal_error( &statement -> error,
                 ERROR_HY010, NULL,
                 statement -> connection -> environment -> requested_version );
-        return function_return( statement, SQL_ERROR );
+
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
     else if ( statement -> state == STATE_S2 &&
             field_identifier != SQL_DESC_COUNT )
@@ -295,7 +345,8 @@ SQLRETURN SQLColAttribute ( SQLHSTMT statement_handle,
         __post_internal_error( &statement -> error,
                 ERROR_07005, NULL,
                 statement -> connection -> environment -> requested_version );
-        return function_return( statement, SQL_ERROR );
+
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
     else if ( statement -> state == STATE_S4 )
     {
@@ -308,7 +359,8 @@ SQLRETURN SQLColAttribute ( SQLHSTMT statement_handle,
         __post_internal_error( &statement -> error,
                 ERROR_24000, NULL,
                 statement -> connection -> environment -> requested_version );
-        return function_return( statement, SQL_ERROR );
+
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
     else if ( statement -> state == STATE_S8 ||
             statement -> state == STATE_S9 ||
@@ -323,7 +375,8 @@ SQLRETURN SQLColAttribute ( SQLHSTMT statement_handle,
         __post_internal_error( &statement -> error,
                 ERROR_HY010, NULL,
                 statement -> connection -> environment -> requested_version );
-        return function_return( statement, SQL_ERROR );
+
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     if ( statement -> state == STATE_S11 ||
@@ -340,7 +393,8 @@ SQLRETURN SQLColAttribute ( SQLHSTMT statement_handle,
             __post_internal_error( &statement -> error,
                     ERROR_HY010, NULL,
                     statement -> connection -> environment -> requested_version );
-            return function_return( statement, SQL_ERROR );
+
+            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
     }
 
@@ -409,6 +463,10 @@ SQLRETURN SQLColAttribute ( SQLHSTMT statement_handle,
                     {
                         unicode_to_ansi_copy( character_attribute, s1,  SQL_NTS, statement -> connection );
                     }
+					if ( SQL_SUCCEEDED( ret ) && string_length ) 
+					{
+						*string_length /= sizeof( SQL_WCHAR );	
+					}
                     break;
 
                   default:
@@ -431,7 +489,8 @@ SQLRETURN SQLColAttribute ( SQLHSTMT statement_handle,
                 __post_internal_error( &statement -> error,
                         ERROR_IM001, NULL,
                         statement -> connection -> environment -> requested_version );
-                return function_return( statement, SQL_ERROR );
+
+                return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
             }
         }
         else
@@ -489,6 +548,10 @@ SQLRETURN SQLColAttribute ( SQLHSTMT statement_handle,
                 {
                     unicode_to_ansi_copy( character_attribute, s1, SQL_NTS, statement -> connection );
                 }
+				if ( SQL_SUCCEEDED( ret ) && string_length ) 
+				{
+					*string_length /= sizeof( SQL_WCHAR );	
+				}
                 break;
 
               default:
@@ -537,7 +600,7 @@ SQLRETURN SQLColAttribute ( SQLHSTMT statement_handle,
                 __post_internal_error( &statement -> error,
                         ERROR_IM001, NULL,
                         statement -> connection -> environment -> requested_version );
-                return function_return( statement, SQL_ERROR );
+                return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
             }
         }
         else
@@ -588,5 +651,5 @@ SQLRETURN SQLColAttribute ( SQLHSTMT statement_handle,
                 statement -> msg );
     }
 
-    return function_return( statement, ret );
+    return function_return( SQL_HANDLE_STMT, statement, ret );
 }
