@@ -27,9 +27,27 @@
  *
  **********************************************************************
  *
- * $Id: SQLColAttributes.c,v 1.7 2002/12/05 17:44:30 lurcher Exp $
+ * $Id: SQLColAttributes.c,v 1.11 2004/11/22 17:02:48 lurcher Exp $
  *
  * $Log: SQLColAttributes.c,v $
+ * Revision 1.11  2004/11/22 17:02:48  lurcher
+ * Fix unicode/ansi conversion in the SQLGet functions
+ *
+ * Revision 1.10  2003/10/30 18:20:45  lurcher
+ *
+ * Fix broken thread protection
+ * Remove SQLNumResultCols after execute, lease S4/S% to driver
+ * Fix string overrun in SQLDriverConnect
+ * Add initial support for Interix
+ *
+ * Revision 1.9  2003/08/15 17:34:43  lurcher
+ *
+ * Remove some unneeded ODBC2->3 attribute conversions
+ *
+ * Revision 1.8  2003/02/27 12:19:39  lurcher
+ *
+ * Add the A functions as well as the W
+ *
  * Revision 1.7  2002/12/05 17:44:30  lurcher
  *
  * Display unknown return values in return logging
@@ -139,7 +157,7 @@
 
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLColAttributes.c,v $ $Revision: 1.7 $";
+static char const rcsid[]= "$RCSfile: SQLColAttributes.c,v $ $Revision: 1.11 $";
 
 SQLINTEGER  map_ca_odbc2_to_3( SQLINTEGER field_identifier )
 {
@@ -149,6 +167,7 @@ SQLINTEGER  map_ca_odbc2_to_3( SQLINTEGER field_identifier )
         field_identifier = SQL_DESC_COUNT; 
         break;
 
+        /*
       case SQL_COLUMN_TYPE:
         field_identifier = SQL_DESC_TYPE; 
         break;
@@ -164,6 +183,7 @@ SQLINTEGER  map_ca_odbc2_to_3( SQLINTEGER field_identifier )
       case SQL_COLUMN_SCALE:
         field_identifier = SQL_DESC_SCALE; 
         break;
+        */
 
       case SQL_COLUMN_NULLABLE:
         field_identifier = SQL_DESC_NULLABLE; 
@@ -180,6 +200,23 @@ SQLINTEGER  map_ca_odbc2_to_3( SQLINTEGER field_identifier )
     return field_identifier;
 }
 
+SQLRETURN SQLColAttributesA( SQLHSTMT statement_handle,
+           SQLUSMALLINT column_number,
+           SQLUSMALLINT field_identifier,
+           SQLPOINTER   character_attribute,
+           SQLSMALLINT  buffer_length,
+           SQLSMALLINT  *string_length,
+           SQLLEN       *numeric_attribute )
+{
+    return SQLColAttributes( statement_handle,
+                            column_number,
+                            field_identifier,
+                            character_attribute,
+                            buffer_length,
+                            string_length,
+                            numeric_attribute );
+}
+
 SQLRETURN SQLColAttributes( SQLHSTMT statement_handle,
            SQLUSMALLINT column_number,
            SQLUSMALLINT field_identifier,
@@ -250,9 +287,7 @@ SQLRETURN SQLColAttributes( SQLHSTMT statement_handle,
                 statement -> connection -> environment -> requested_version,
                 SQL_API_SQLCOLATTRIBUTES );
 
-        thread_release( SQL_HANDLE_STMT, statement );
-
-        return function_return( statement, SQL_ERROR );
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
 	/*
@@ -273,9 +308,7 @@ SQLRETURN SQLColAttributes( SQLHSTMT statement_handle,
                 ERROR_07009, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_STMT, statement );
-
-        return function_return( statement, SQL_ERROR );
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 	*/
 
@@ -294,9 +327,7 @@ SQLRETURN SQLColAttributes( SQLHSTMT statement_handle,
                 ERROR_HY010, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_STMT, statement );
-
-        return function_return( statement, SQL_ERROR );
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
     else if ( statement -> state == STATE_S2 &&
             field_identifier != SQL_DESC_COUNT )
@@ -311,9 +342,7 @@ SQLRETURN SQLColAttributes( SQLHSTMT statement_handle,
                 ERROR_07005, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_STMT, statement );
-
-        return function_return( statement, SQL_ERROR );
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
     else if ( statement -> state == STATE_S4 )
     {
@@ -327,9 +356,7 @@ SQLRETURN SQLColAttributes( SQLHSTMT statement_handle,
                 ERROR_24000, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_STMT, statement );
-
-        return function_return( statement, SQL_ERROR );
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
     else if ( statement -> state == STATE_S8 ||
             statement -> state == STATE_S9 ||
@@ -345,9 +372,7 @@ SQLRETURN SQLColAttributes( SQLHSTMT statement_handle,
                 ERROR_HY010, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_STMT, statement );
-
-        return function_return( statement, SQL_ERROR );
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     if ( statement -> state == STATE_S11 ||
@@ -365,9 +390,7 @@ SQLRETURN SQLColAttributes( SQLHSTMT statement_handle,
                     ERROR_HY010, NULL,
                     statement -> connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_STMT, statement );
-
-            return function_return( statement, SQL_ERROR );
+            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
     }
 
@@ -403,6 +426,10 @@ SQLRETURN SQLColAttributes( SQLHSTMT statement_handle,
                 {
                     unicode_to_ansi_copy( character_attribute, s1, SQL_NTS, statement -> connection );
                 }
+				if ( SQL_SUCCEEDED( ret ) && string_length && character_attribute ) 
+				{
+					*string_length /= sizeof( SQL_WCHAR );	
+				}
 
                 if ( s1 )
                 {
@@ -421,9 +448,7 @@ SQLRETURN SQLColAttributes( SQLHSTMT statement_handle,
                         ERROR_IM001, NULL,
                         statement -> connection -> environment -> requested_version );
 
-                thread_release( SQL_HANDLE_STMT, statement );
-
-                return function_return( statement, SQL_ERROR );
+                return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
             }
         }
         else
@@ -447,7 +472,11 @@ SQLRETURN SQLColAttributes( SQLHSTMT statement_handle,
             if ( SQL_SUCCEEDED( ret ) && character_attribute )
             {
                 unicode_to_ansi_copy( character_attribute, s1, SQL_NTS, statement -> connection );
-            }
+			}
+			if ( SQL_SUCCEEDED( ret ) && string_length && character_attribute ) 
+			{
+				*string_length /= sizeof( SQL_WCHAR );	
+			}
 
             if ( s1 )
             {
@@ -488,9 +517,7 @@ SQLRETURN SQLColAttributes( SQLHSTMT statement_handle,
                         ERROR_IM001, NULL,
                         statement -> connection -> environment -> requested_version );
 
-                thread_release( SQL_HANDLE_STMT, statement );
-
-                return function_return( statement, SQL_ERROR );
+                return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
             }
         }
         else
@@ -560,7 +587,5 @@ SQLRETURN SQLColAttributes( SQLHSTMT statement_handle,
                 statement -> msg );
     }
 
-    thread_release( SQL_HANDLE_STMT, statement );
-
-    return function_return( statement, ret );
+    return function_return( SQL_HANDLE_STMT, statement, ret );
 }
