@@ -27,9 +27,39 @@
  *
  **********************************************************************
  *
- * $Id: SQLDriverConnect.c,v 1.12 2003/01/23 15:33:24 lurcher Exp $
+ * $Id: SQLDriverConnect.c,v 1.19 2004/10/22 09:10:19 lurcher Exp $
  *
  * $Log: SQLDriverConnect.c,v $
+ * Revision 1.19  2004/10/22 09:10:19  lurcher
+ * Fix a couple of problems with FILEDSN's
+ *
+ * Revision 1.18  2004/09/08 16:38:54  lurcher
+ *
+ * Get ready for a 2.2.10 release
+ *
+ * Revision 1.17  2004/07/27 13:04:41  lurcher
+ *
+ * Strip FILEDSN from connection string before passing to driver
+ *
+ * Revision 1.16  2004/02/18 15:47:44  lurcher
+ *
+ * Fix a leak in the iconv code
+ *
+ * Revision 1.15  2003/10/30 18:20:45  lurcher
+ *
+ * Fix broken thread protection
+ * Remove SQLNumResultCols after execute, lease S4/S% to driver
+ * Fix string overrun in SQLDriverConnect
+ * Add initial support for Interix
+ *
+ * Revision 1.14  2003/09/08 15:34:29  lurcher
+ *
+ * A couple of small but perfectly formed fixes
+ *
+ * Revision 1.13  2003/02/27 12:19:39  lurcher
+ *
+ * Add the A functions as well as the W
+ *
  * Revision 1.12  2003/01/23 15:33:24  lurcher
  *
  * Fix problems with using putenv()
@@ -252,7 +282,7 @@
 #include <string.h>
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLDriverConnect.c,v $ $Revision: 1.12 $";
+static char const rcsid[]= "$RCSfile: SQLDriverConnect.c,v $ $Revision: 1.19 $";
 
 /*
  * connection pooling stuff
@@ -309,7 +339,12 @@ int len;
     if ( !**cp )
         return;
 
-    while ( **cp && **cp != '=' )
+    /* 
+     * To handle the case attribute in which the attribute is of the form
+     * "ATTR;" instead of "ATTR=VALUE;"
+     */
+
+    while ( **cp && **cp != ';' && **cp != '=' )
     {
         (*cp)++;
     }
@@ -322,7 +357,7 @@ int len;
     memcpy( *keyword, ptr, len );
     (*keyword)[ len ] = '\0';
 
-    (*cp)++;
+    (**cp != ';') && (*cp)++;
     ptr = *cp;
 
     if ( strcmp( *keyword, "DRIVER" ) == 0 )
@@ -549,6 +584,26 @@ void __release_conn( struct con_struct *con_str )
     con_str -> count = 0;
 }
 
+SQLRETURN SQLDriverConnectA(
+    SQLHDBC            hdbc,
+    SQLHWND            hwnd,
+    SQLCHAR            *conn_str_in,
+    SQLSMALLINT        len_conn_str_in,
+    SQLCHAR            *conn_str_out,
+    SQLSMALLINT        conn_str_out_max,
+    SQLSMALLINT        *ptr_conn_str_out,
+    SQLUSMALLINT       driver_completion )
+{
+    return SQLDriverConnect( hdbc,
+                                hwnd,
+                                conn_str_in,
+                                len_conn_str_in,
+                                conn_str_out,
+                                conn_str_out_max,
+                                ptr_conn_str_out,
+                                driver_completion );
+}
+
 SQLRETURN SQLDriverConnect(
     SQLHDBC            hdbc,
     SQLHWND            hwnd,
@@ -568,6 +623,7 @@ SQLRETURN SQLDriverConnect(
     SQLCHAR s1[ 2048 ];
     SQLCHAR local_conn_str_in[ 2048 ];
     SQLCHAR local_out_conection[ 2048 ];
+    char save_filedsn[ 128 ];
     int warnings;
 
     /*
@@ -639,9 +695,7 @@ SQLRETURN SQLDriverConnect(
                 ERROR_HY090, NULL,
                 connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     if ( driver_completion == SQL_DRIVER_PROMPT &&
@@ -657,9 +711,7 @@ SQLRETURN SQLDriverConnect(
                 ERROR_HY092, NULL,
                 connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     if ( driver_completion != SQL_DRIVER_PROMPT &&
@@ -677,9 +729,7 @@ SQLRETURN SQLDriverConnect(
                 ERROR_HY110, NULL,
                 connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     /*
@@ -698,9 +748,7 @@ SQLRETURN SQLDriverConnect(
                 ERROR_08002, NULL,
                 connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     /*
@@ -744,8 +792,8 @@ SQLRETURN SQLDriverConnect(
 
             if ( len_conn_str_in >= conn_str_out_max )
             {
-                memcpy( conn_str_out, conn_str_in, conn_str_out_max );
-                conn_str_out[ conn_str_out_max ] = '\0';
+                memcpy( conn_str_out, conn_str_in, conn_str_out_max - 1 );
+                conn_str_out[ conn_str_out_max - 1 ] = '\0';
                 if ( ptr_conn_str_out )
                 {
                     *ptr_conn_str_out = len_conn_str_in;
@@ -783,9 +831,7 @@ SQLRETURN SQLDriverConnect(
 
         connection -> state = STATE_C4;
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, ret_from_connect );
+        return function_return( SQL_HANDLE_DBC, connection, ret_from_connect );
     }
 
     /*
@@ -818,6 +864,20 @@ SQLRETURN SQLDriverConnect(
     __parse_connection_string( &con_struct,
             (char*)conn_str_in, len_conn_str_in );
 
+    /*
+     * get for later
+     */
+
+    tsavefile = __get_attribute_value( &con_struct, "SAVEFILE" );
+    if ( tsavefile )
+    {
+        strcpy( savefile, tsavefile );
+    }
+    else
+    {
+        savefile[ 0 ] = '\0';
+    }
+
     /*
      * open the file dsn, get each entry from it, if its not in the connection 
      * struct, add it
@@ -832,6 +892,8 @@ SQLRETURN SQLDriverConnect(
         {
             struct con_struct con_struct1;
 
+	    	strcpy( save_filedsn, filedsn );
+
             if ( strlen( str ))
             {
                 strcpy((char*)local_conn_str_in, (char*)conn_str_in );
@@ -840,6 +902,43 @@ SQLRETURN SQLDriverConnect(
                 __parse_connection_string( &con_struct1,
                         str, strlen( str ));
 
+				/*
+		 		* Get the attributes from the original string
+		 		*/
+
+				conn_str_in[ 0 ] = '\0';
+
+                if ( con_struct.count )
+                {
+                    struct con_pair *cp;
+
+                    cp = con_struct.list;
+                    while( cp )
+                    {
+		    			char str1[ 256 ];
+
+						/*
+			 			* Don't pass FILEDSN down
+			 			*/
+
+						if ( strcmp( cp -> keyword, "FILEDSN" ) &&
+							strcmp( cp -> keyword, "FILEDSN" ) )
+						{
+							if ( strlen( conn_str_in ) > 0 )
+							{
+                				sprintf( str1, ";%s=%s", cp -> keyword, cp -> attribute );
+							}
+							else
+							{
+                   				sprintf( str1, "%s=%s", cp -> keyword, cp -> attribute );
+							}
+                			strcat((char*) conn_str_in, str1 );
+						}
+
+                        cp = cp -> next;
+                    }
+                }
+
                 if ( con_struct1.count )
                 {
                     struct con_pair *cp;
@@ -851,7 +950,14 @@ SQLRETURN SQLDriverConnect(
                         {
                             char str1[ 256 ];
 
-                            sprintf( str1, ";%s=%s", cp -> keyword, cp -> attribute );
+							if ( strlen( conn_str_in ) > 0 )
+							{
+                            	sprintf( str1, ";%s=%s", cp -> keyword, cp -> attribute );
+							}
+							else
+							{
+                            	sprintf( str1, "%s=%s", cp -> keyword, cp -> attribute );
+							}
                             strcat((char*) conn_str_in, str1 );
                         }
                         cp = cp -> next;
@@ -873,6 +979,10 @@ SQLRETURN SQLDriverConnect(
                     (char*)conn_str_in, len_conn_str_in );
         }
     }
+    else
+    {
+	    save_filedsn[ 0 ] = '\0';
+    }
 
     /*
      * look for some keywords
@@ -889,8 +999,19 @@ SQLRETURN SQLDriverConnect(
 
         strcpy( driver_name, driver );
 
+#ifdef PLATFORM64
+        SQLGetPrivateProfileString( driver, "Driver64", "",
+                lib_name, sizeof( lib_name ), "ODBCINST.INI" );
+
+		if ( lib_name[ 0 ] == '\0' )
+		{
+        	SQLGetPrivateProfileString( driver, "Driver", "",
+                	lib_name, sizeof( lib_name ), "ODBCINST.INI" );
+		}
+#else
         SQLGetPrivateProfileString( driver, "Driver", "",
                 lib_name, sizeof( lib_name ), "ODBCINST.INI" );
+#endif
 
         if ( lib_name[ 0 ] == '\0' )
         {
@@ -911,9 +1032,7 @@ SQLRETURN SQLDriverConnect(
                     connection -> environment -> requested_version );
             __release_conn( &con_struct );
 
-            thread_release( SQL_HANDLE_DBC, connection );
-
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
 
         strcpy( connection -> dsn, "" );
@@ -936,9 +1055,7 @@ SQLRETURN SQLDriverConnect(
 
             __release_conn( &con_struct );
 
-            thread_release( SQL_HANDLE_DBC, connection );
-
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
 
         if ( strlen( dsn ) > SQL_MAX_DSN_LENGTH )
@@ -955,9 +1072,7 @@ SQLRETURN SQLDriverConnect(
 
             __release_conn( &con_struct );
 
-            thread_release( SQL_HANDLE_DBC, connection );
-
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
 
         /*
@@ -978,24 +1093,13 @@ SQLRETURN SQLDriverConnect(
 
             __release_conn( &con_struct );
 
-            thread_release( SQL_HANDLE_DBC, connection );
-
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
 
         strcpy( connection -> dsn, dsn );
         __handle_attr_extensions( connection, dsn, driver_name );
     }
 
-    /*
-     * get for later
-     */
-
-    tsavefile = __get_attribute_value( &con_struct, "SAVEFILE" );
-    if ( tsavefile )
-    {
-        strcpy( savefile, tsavefile );
-    }
 
     if ( dsn )
     {
@@ -1013,8 +1117,7 @@ SQLRETURN SQLDriverConnect(
      */
     if ( !__connect_part_one( connection, lib_name, driver_name, &warnings ))
     {
-        thread_release( SQL_HANDLE_DBC, connection );
-        return function_return( connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     if ( !CHECK_SQLDRIVERCONNECT( connection ) &&
@@ -1031,9 +1134,7 @@ SQLRETURN SQLDriverConnect(
                 ERROR_IM001, NULL,
                 connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     if ( CHECK_SQLDRIVERCONNECT( connection ))
@@ -1147,8 +1248,7 @@ SQLRETURN SQLDriverConnect(
         if ( !SQL_SUCCEEDED( ret_from_connect ))
         {
             __disconnect_part_one( connection );
-
-            thread_release( SQL_HANDLE_DBC, connection );
+            __disconnect_part_four( connection );
 
             sprintf( connection -> msg,
                     "\n\t\tExit:[%s]",
@@ -1160,7 +1260,7 @@ SQLRETURN SQLDriverConnect(
                     LOG_INFO,
                     connection -> msg );
 
-            return function_return( connection, ret_from_connect );
+            return function_return( SQL_HANDLE_DBC, connection, ret_from_connect );
         }
         connection -> unicode_driver = 0;
     }
@@ -1302,8 +1402,7 @@ SQLRETURN SQLDriverConnect(
         if ( !SQL_SUCCEEDED( ret_from_connect ))
         {
             __disconnect_part_one( connection );
-
-            thread_release( SQL_HANDLE_DBC, connection );
+            __disconnect_part_four( connection );
 
             sprintf( connection -> msg,
                     "\n\t\tExit:[%s]",
@@ -1315,7 +1414,7 @@ SQLRETURN SQLDriverConnect(
                     LOG_INFO,
                     connection -> msg );
 
-            return function_return( connection, ret_from_connect );
+            return function_return( SQL_HANDLE_DBC, connection, ret_from_connect );
         }
         else
         {
@@ -1359,10 +1458,9 @@ SQLRETURN SQLDriverConnect(
     {
         __disconnect_part_two( connection );
         __disconnect_part_one( connection );
+        __disconnect_part_four( connection );
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     if ( log_info.log_flag )
@@ -1392,7 +1490,40 @@ SQLRETURN SQLDriverConnect(
                 connection -> msg );
     }
 
-    thread_release( SQL_HANDLE_DBC, connection );
+    /*
+     * If we specified FILEDSN or SAVEFILE these need adding to the
+     * output string
+     */
+
+    if ( strlen( savefile ))
+    {
+	    char *str = strdup( conn_str_out );
+	    strcpy( conn_str_out, "SAVEFILE=" );
+	    strcat( conn_str_out, savefile );
+	    strcat( conn_str_out, ";" );
+	    strcat( conn_str_out, str );
+	    free( str );
+
+	    if ( ptr_conn_str_out ) 
+	    {
+		    *ptr_conn_str_out = strlen( conn_str_out );
+	    }
+    }
+
+    if ( strlen( save_filedsn ))
+    {
+	    char *str = strdup( conn_str_out );
+	    strcpy( conn_str_out, "FILEDSN=" );
+	    strcat( conn_str_out, save_filedsn );
+	    strcat( conn_str_out, ";" );
+	    strcat( conn_str_out, str );
+	    free( str );
+
+	    if ( ptr_conn_str_out ) 
+	    {
+		    *ptr_conn_str_out = strlen( conn_str_out );
+	    }
+    }
 
     /*
      * write the connection string out to a file
@@ -1424,6 +1555,15 @@ SQLRETURN SQLDriverConnect(
                         /*
                          * don't save this
                          */
+                    	cp = cp -> next;
+                        continue;
+                    }
+		    else if ( strcasecmp( cp -> keyword, "SAVEFILE" ) == 0 )
+                    {
+                        /*
+                         * or this
+                         */
+                    	cp = cp -> next;
                         continue;
                     }
                     else if ( strcasecmp( cp -> keyword, "DSN" ) == 0 )
@@ -1432,6 +1572,7 @@ SQLRETURN SQLDriverConnect(
                          * don't save this either, there should be enough with the added DRIVER=
                          * to make it work
                          */
+                    	cp = cp -> next;
                         continue;
                     }
                     else if ( strcasecmp( cp -> keyword, "DRIVER" ) == 0 )
@@ -1457,5 +1598,5 @@ SQLRETURN SQLDriverConnect(
         ret_from_connect = SQL_SUCCESS_WITH_INFO;
     }
 
-    return function_return( connection, ret_from_connect );
+    return function_return( SQL_HANDLE_DBC, connection, ret_from_connect );
 }
