@@ -27,9 +27,20 @@
  *
  **********************************************************************
  *
- * $Id: SQLDriverConnectW.c,v 1.11 2002/12/20 11:36:46 lurcher Exp $
+ * $Id: SQLDriverConnectW.c,v 1.13 2003/10/30 18:20:45 lurcher Exp $
  *
  * $Log: SQLDriverConnectW.c,v $
+ * Revision 1.13  2003/10/30 18:20:45  lurcher
+ *
+ * Fix broken thread protection
+ * Remove SQLNumResultCols after execute, lease S4/S% to driver
+ * Fix string overrun in SQLDriverConnect
+ * Add initial support for Interix
+ *
+ * Revision 1.12  2003/08/08 11:14:21  lurcher
+ *
+ * Fix UNICODE problem in SQLDriverConnectW
+ *
  * Revision 1.11  2002/12/20 11:36:46  lurcher
  *
  * Update DMEnvAttr code to allow setting in the odbcinst.ini entry
@@ -252,9 +263,7 @@ SQLRETURN SQLDriverConnectW(
                 ERROR_HY090, NULL,
                 connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     if ( driver_completion == SQL_DRIVER_PROMPT &&
@@ -270,9 +279,7 @@ SQLRETURN SQLDriverConnectW(
                 ERROR_HY092, NULL,
                 connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     if ( driver_completion != SQL_DRIVER_PROMPT &&
@@ -290,9 +297,7 @@ SQLRETURN SQLDriverConnectW(
                 ERROR_HY110, NULL,
                 connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     /*
@@ -311,9 +316,7 @@ SQLRETURN SQLDriverConnectW(
                 ERROR_08002, NULL,
                 connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     /*
@@ -379,9 +382,7 @@ SQLRETURN SQLDriverConnectW(
                     connection -> environment -> requested_version );
             __release_conn( &con_struct );
 
-            thread_release( SQL_HANDLE_DBC, connection );
-
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
 
         strcpy( connection -> dsn, "" );
@@ -404,9 +405,7 @@ SQLRETURN SQLDriverConnectW(
                     connection -> environment -> requested_version );
             __release_conn( &con_struct );
 
-            thread_release( SQL_HANDLE_DBC, connection );
-
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
 
         if ( strlen( dsn ) > SQL_MAX_DSN_LENGTH )
@@ -421,9 +420,7 @@ SQLRETURN SQLDriverConnectW(
                     ERROR_IM012, NULL,
                     connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_DBC, connection );
-
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
 
         /*
@@ -443,9 +440,7 @@ SQLRETURN SQLDriverConnectW(
                     connection -> environment -> requested_version );
             __release_conn( &con_struct );
 
-            thread_release( SQL_HANDLE_DBC, connection );
-
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
 
         strcpy( connection -> dsn, dsn );
@@ -468,8 +463,7 @@ SQLRETURN SQLDriverConnectW(
      */
     if ( !__connect_part_one( connection, lib_name, driver_name, &warnings ))
     {
-        thread_release( SQL_HANDLE_DBC, connection );
-        return function_return( connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     if ( !CHECK_SQLDRIVERCONNECTW( connection ) &&
@@ -486,9 +480,7 @@ SQLRETURN SQLDriverConnectW(
                 ERROR_IM001, NULL,
                 connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     if ( CHECK_SQLDRIVERCONNECTW( connection ))
@@ -514,7 +506,7 @@ SQLRETURN SQLDriverConnectW(
                 ptr_conn_str_out,
                 driver_completion );
 
-        if ( ret_from_connect != SQL_ERROR )
+        if ( ret_from_connect != SQL_SUCCESS )
         {
             SQLWCHAR sqlstate[ 6 ];
             SQLINTEGER native_error;
@@ -591,8 +583,6 @@ SQLRETURN SQLDriverConnectW(
             {
                 __disconnect_part_one( connection );
 
-                thread_release( SQL_HANDLE_DBC, connection );
-
                 sprintf( connection -> msg,
                         "\n\t\tExit:[%s]",
                             __get_return_status( ret_from_connect, s1 ));
@@ -603,7 +593,7 @@ SQLRETURN SQLDriverConnectW(
                         LOG_INFO,
                         connection -> msg );
 
-                return function_return( connection, ret_from_connect );
+                return function_return( SQL_HANDLE_DBC, connection, ret_from_connect );
             }
         }
         connection -> unicode_driver = 1;
@@ -739,8 +729,6 @@ SQLRETURN SQLDriverConnectW(
             {
                 __disconnect_part_one( connection );
 
-                thread_release( SQL_HANDLE_DBC, connection );
-
                 sprintf( connection -> msg,
                         "\n\t\tExit:[%s]",
                             __get_return_status( ret_from_connect, s1 ));
@@ -751,7 +739,7 @@ SQLRETURN SQLDriverConnectW(
                         LOG_INFO,
                         connection -> msg );
 
-                return function_return( connection, ret_from_connect );
+                return function_return( SQL_HANDLE_DBC, connection, ret_from_connect );
             }
         }
         connection -> unicode_driver = 0;
@@ -784,9 +772,7 @@ SQLRETURN SQLDriverConnectW(
         __disconnect_part_two( connection );
         __disconnect_part_one( connection );
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     if ( log_info.log_flag )
@@ -819,12 +805,10 @@ SQLRETURN SQLDriverConnectW(
                 connection -> msg );
     }
 
-    thread_release( SQL_HANDLE_DBC, connection );
-
     if ( warnings && ret_from_connect == SQL_SUCCESS )
     {
         ret_from_connect = SQL_SUCCESS_WITH_INFO;
     }
 
-    return function_return( connection, ret_from_connect );
+    return function_return( SQL_HANDLE_DBC, connection, ret_from_connect );
 }
