@@ -27,9 +27,13 @@
  *
  **********************************************************************
  *
- * $Id: SQLError.c,v 1.7 2002/12/05 17:44:30 lurcher Exp $
+ * $Id: SQLError.c,v 1.8 2003/02/27 12:19:39 lurcher Exp $
  *
  * $Log: SQLError.c,v $
+ * Revision 1.8  2003/02/27 12:19:39  lurcher
+ *
+ * Add the A functions as well as the W
+ *
  * Revision 1.7  2002/12/05 17:44:30  lurcher
  *
  * Display unknown return values in return logging
@@ -168,7 +172,7 @@
 
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLError.c,v $ $Revision: 1.7 $";
+static char const rcsid[]= "$RCSfile: SQLError.c,v $ $Revision: 1.8 $";
 
 static SQLRETURN extract_sql_error( EHEAD *head,
         SQLCHAR *sqlstate,
@@ -268,6 +272,25 @@ static SQLRETURN extract_sql_error( EHEAD *head,
     return ret;
 }
 
+SQLRETURN SQLErrorA( SQLHENV environment_handle,
+           SQLHDBC connection_handle,
+           SQLHSTMT statement_handle,
+           SQLCHAR *sqlstate,
+           SQLINTEGER *native_error,
+           SQLCHAR *message_text,
+           SQLSMALLINT buffer_length,
+           SQLSMALLINT *text_length )
+{
+    return SQLError( environment_handle,
+                        connection_handle,
+                        statement_handle,
+                        sqlstate,
+                        native_error,
+                        message_text,
+                        buffer_length,
+                        text_length );
+}
+
 SQLRETURN SQLError( SQLHENV environment_handle,
            SQLHDBC connection_handle,
            SQLHSTMT statement_handle,
