@@ -27,9 +27,23 @@
  *
  **********************************************************************
  *
- * $Id: SQLGetConnectAttr.c,v 1.8 2002/12/05 17:44:30 lurcher Exp $
+ * $Id: SQLGetConnectAttr.c,v 1.11 2004/11/22 17:02:48 lurcher Exp $
  *
  * $Log: SQLGetConnectAttr.c,v $
+ * Revision 1.11  2004/11/22 17:02:48  lurcher
+ * Fix unicode/ansi conversion in the SQLGet functions
+ *
+ * Revision 1.10  2003/10/30 18:20:45  lurcher
+ *
+ * Fix broken thread protection
+ * Remove SQLNumResultCols after execute, lease S4/S% to driver
+ * Fix string overrun in SQLDriverConnect
+ * Add initial support for Interix
+ *
+ * Revision 1.9  2003/02/27 12:19:39  lurcher
+ *
+ * Add the A functions as well as the W
+ *
  * Revision 1.8  2002/12/05 17:44:30  lurcher
  *
  * Display unknown return values in return logging
@@ -137,7 +151,20 @@
 
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLGetConnectAttr.c,v $ $Revision: 1.8 $";
+static char const rcsid[]= "$RCSfile: SQLGetConnectAttr.c,v $ $Revision: 1.11 $";
+
+SQLRETURN SQLGetConnectAttrA( SQLHDBC connection_handle,
+           SQLINTEGER attribute,
+           SQLPOINTER value,
+           SQLINTEGER buffer_length,
+           SQLINTEGER *string_length )
+{
+    return SQLGetConnectAttr( connection_handle,
+                        attribute,
+                        value,
+                        buffer_length,
+                        string_length );
+}
 
 SQLRETURN SQLGetConnectAttr( SQLHDBC connection_handle,
            SQLINTEGER attribute,
@@ -269,9 +296,7 @@ SQLRETURN SQLGetConnectAttr( SQLHDBC connection_handle,
                 ERROR_HY010, NULL,
                 connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     if ( connection -> state == STATE_C2 )
@@ -304,9 +329,7 @@ SQLRETURN SQLGetConnectAttr( SQLHDBC connection_handle,
                     ERROR_08003, NULL,
                     connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_DBC, connection );
-
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
     }
 
@@ -526,9 +549,7 @@ SQLRETURN SQLGetConnectAttr( SQLHDBC connection_handle,
                 LOG_INFO, 
                 connection -> msg );
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, ret );
+        return function_return( SQL_HANDLE_DBC, connection, ret );
     }
     else
     {
@@ -562,9 +583,7 @@ SQLRETURN SQLGetConnectAttr( SQLHDBC connection_handle,
                                 ERROR_HY092, NULL,
                                 connection -> environment -> requested_version );
 
-                        thread_release( SQL_HANDLE_STMT, connection );
-
-                        return function_return( connection, SQL_ERROR );
+                        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
                     }
 
                     switch( attribute )
@@ -593,6 +612,10 @@ SQLRETURN SQLGetConnectAttr( SQLHDBC connection_handle,
                         {
                             unicode_to_ansi_copy( value, s1, SQL_NTS, connection );
                         }
+						if ( SQL_SUCCEEDED( ret ) && string_length ) 
+						{
+							*string_length /= sizeof( SQL_WCHAR );	
+						}
                         break;
                     }
 
@@ -613,9 +636,7 @@ SQLRETURN SQLGetConnectAttr( SQLHDBC connection_handle,
                             ERROR_IM001, NULL,
                             connection -> environment -> requested_version );
 
-                    thread_release( SQL_HANDLE_DBC, connection );
-
-                    return function_return( connection, SQL_ERROR );
+                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
                 }
             }
             else
@@ -650,6 +671,10 @@ SQLRETURN SQLGetConnectAttr( SQLHDBC connection_handle,
                     {
                         unicode_to_ansi_copy( value, s1, SQL_NTS, connection );
                     }
+					if ( SQL_SUCCEEDED( ret ) && string_length ) 
+					{
+						*string_length /= sizeof( SQL_WCHAR );	
+					}
                     break;
                 }
 
@@ -682,9 +707,7 @@ SQLRETURN SQLGetConnectAttr( SQLHDBC connection_handle,
                                 ERROR_HY092, NULL,
                                 connection -> environment -> requested_version );
 
-                        thread_release( SQL_HANDLE_STMT, connection );
-
-                        return function_return( connection, SQL_ERROR );
+                        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
                     }
 
                     ret = SQLGETCONNECTOPTION( connection,
@@ -704,9 +727,7 @@ SQLRETURN SQLGetConnectAttr( SQLHDBC connection_handle,
                             ERROR_IM001, NULL,
                             connection -> environment -> requested_version );
 
-                    thread_release( SQL_HANDLE_DBC, connection );
-
-                    return function_return( connection, SQL_ERROR );
+                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
                 }
             }
             else
@@ -733,8 +754,7 @@ SQLRETURN SQLGetConnectAttr( SQLHDBC connection_handle,
                     connection -> msg );
         }
 
-        thread_release( SQL_HANDLE_DBC, connection );
 
-        return function_return( connection, ret );
+        return function_return( SQL_HANDLE_DBC, connection, ret );
     }
 }
