@@ -27,9 +27,19 @@
  *
  **********************************************************************
  *
- * $Id: SQLGetConnectAttrW.c,v 1.9 2002/12/05 17:44:30 lurcher Exp $
+ * $Id: SQLGetConnectAttrW.c,v 1.11 2004/11/22 17:02:48 lurcher Exp $
  *
  * $Log: SQLGetConnectAttrW.c,v $
+ * Revision 1.11  2004/11/22 17:02:48  lurcher
+ * Fix unicode/ansi conversion in the SQLGet functions
+ *
+ * Revision 1.10  2003/10/30 18:20:45  lurcher
+ *
+ * Fix broken thread protection
+ * Remove SQLNumResultCols after execute, lease S4/S% to driver
+ * Fix string overrun in SQLDriverConnect
+ * Add initial support for Interix
+ *
  * Revision 1.9  2002/12/05 17:44:30  lurcher
  *
  * Display unknown return values in return logging
@@ -233,9 +243,7 @@ SQLRETURN SQLGetConnectAttrW( SQLHDBC connection_handle,
                 ERROR_HY010, NULL,
                 connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     if ( connection -> state == STATE_C2 )
@@ -268,9 +276,7 @@ SQLRETURN SQLGetConnectAttrW( SQLHDBC connection_handle,
                     ERROR_08003, NULL,
                     connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_DBC, connection );
-
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
     }
 
@@ -490,9 +496,7 @@ SQLRETURN SQLGetConnectAttrW( SQLHDBC connection_handle,
                 LOG_INFO, 
                 connection -> msg );
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, ret );
+        return function_return( SQL_HANDLE_DBC, connection, ret );
     }
     else
     {
@@ -524,9 +528,7 @@ SQLRETURN SQLGetConnectAttrW( SQLHDBC connection_handle,
                                 ERROR_HY092, NULL,
                                 connection -> environment -> requested_version );
 
-                        thread_release( SQL_HANDLE_STMT, connection );
-
-                        return function_return( connection, SQL_ERROR );
+                        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
                     }
 
                     ret = SQLGETCONNECTOPTIONW( connection,
@@ -546,9 +548,7 @@ SQLRETURN SQLGetConnectAttrW( SQLHDBC connection_handle,
                             ERROR_IM001, NULL,
                             connection -> environment -> requested_version );
 
-                    thread_release( SQL_HANDLE_DBC, connection );
-
-                    return function_return( connection, SQL_ERROR );
+                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
                 }
             }
             else
@@ -586,9 +586,7 @@ SQLRETURN SQLGetConnectAttrW( SQLHDBC connection_handle,
                                 ERROR_HY092, NULL,
                                 connection -> environment -> requested_version );
 
-                        thread_release( SQL_HANDLE_STMT, connection );
-
-                        return function_return( connection, SQL_ERROR );
+                        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
                     }
 
                     switch( attribute )
@@ -617,11 +615,14 @@ SQLRETURN SQLGetConnectAttrW( SQLHDBC connection_handle,
                         {
                             ansi_to_unicode_copy( value, (char*) as1, SQL_NTS, connection );
                         }
-
                         if ( as1 )
                         {
                             free( as1 );
                         }
+						if ( SQL_SUCCEEDED( ret ) && string_length )
+						{
+							*string_length *= sizeof( SQLWCHAR );
+						}
                         break;
                     }
                 }
@@ -637,9 +638,7 @@ SQLRETURN SQLGetConnectAttrW( SQLHDBC connection_handle,
                             ERROR_IM001, NULL,
                             connection -> environment -> requested_version );
 
-                    thread_release( SQL_HANDLE_DBC, connection );
-
-                    return function_return( connection, SQL_ERROR );
+                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
                 }
             }
             else
@@ -675,11 +674,14 @@ SQLRETURN SQLGetConnectAttrW( SQLHDBC connection_handle,
                     {
                         ansi_to_unicode_copy( value, (char*)as1, SQL_NTS, connection );
                     }
-
                     if ( as1 )
                     {
                         free( as1 );
-                    }
+                    }		
+					if ( SQL_SUCCEEDED( ret ) && string_length )
+					{
+						*string_length *= sizeof( SQLWCHAR );
+					}
                     break;
                 }
             }
@@ -698,8 +700,6 @@ SQLRETURN SQLGetConnectAttrW( SQLHDBC connection_handle,
                     connection -> msg );
         }
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, ret );
+        return function_return( SQL_HANDLE_DBC, connection, ret );
     }
 }
