@@ -27,9 +27,19 @@
  *
  **********************************************************************
  *
- * $Id: SQLGetDiagRec.c,v 1.9 2002/12/05 17:44:31 lurcher Exp $
+ * $Id: SQLGetDiagRec.c,v 1.11 2003/02/27 12:19:39 lurcher Exp $
  *
  * $Log: SQLGetDiagRec.c,v $
+ * Revision 1.11  2003/02/27 12:19:39  lurcher
+ *
+ * Add the A functions as well as the W
+ *
+ * Revision 1.10  2003/02/25 13:28:30  lurcher
+ *
+ * Allow errors on the drivers AllocHandle to be reported
+ * Fix a problem that caused errors to not be reported in the log
+ * Remove a redundant line from the spec file
+ *
  * Revision 1.9  2002/12/05 17:44:31  lurcher
  *
  * Display unknown return values in return logging
@@ -142,7 +152,7 @@
 
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLGetDiagRec.c,v $ $Revision: 1.9 $";
+static char const rcsid[]= "$RCSfile: SQLGetDiagRec.c,v $ $Revision: 1.11 $";
 
 DMHDBC __get_connection( EHEAD * head )
 {
@@ -467,6 +477,25 @@ static SQLRETURN extract_sql_error_rec( EHEAD *head,
     }
 }
 
+SQLRETURN SQLGetDiagRecA( SQLSMALLINT handle_type,
+        SQLHANDLE   handle,
+        SQLSMALLINT rec_number,
+        SQLCHAR     *sqlstate,
+        SQLINTEGER  *native,
+        SQLCHAR     *message_text,
+        SQLSMALLINT buffer_length,
+        SQLSMALLINT *text_length_ptr )
+{
+    return SQLGetDiagRec( handle_type,
+                        handle,
+                        rec_number,
+                        sqlstate,
+                        native,
+                        message_text,
+                        buffer_length,
+                        text_length_ptr );
+}
+
 SQLRETURN SQLGetDiagRec( SQLSMALLINT handle_type,
         SQLHANDLE   handle,
         SQLSMALLINT rec_number,
@@ -545,7 +574,7 @@ SQLRETURN SQLGetDiagRec( SQLSMALLINT handle_type,
                     \n\t\t\tSQLState = %s\
                     \n\t\t\tNative = %s\
                     \n\t\t\tMessage Text = %s",
-                        __get_return_status( ret, s1 ),
+                        __get_return_status( ret, s2 ),
                         sqlstate,
                         __ptr_as_string( s0, native ),
                         __sdata_as_string( s1, SQL_CHAR, 
