@@ -27,9 +27,31 @@
  *
  **********************************************************************
  *
- * $Id: SQLSetConnectAttr.c,v 1.10 2002/12/05 17:44:31 lurcher Exp $
+ * $Id: SQLSetConnectAttr.c,v 1.15 2005/03/01 15:50:50 lurcher Exp $
  *
  * $Log: SQLSetConnectAttr.c,v $
+ * Revision 1.15  2005/03/01 15:50:50  lurcher
+ * Add Eric's SQLSetConnectAttr patch
+ *
+ * Revision 1.14  2004/06/21 10:01:11  lurcher
+ *
+ * Fix a couple of 64 bit issues
+ *
+ * Revision 1.13  2003/10/30 18:20:46  lurcher
+ *
+ * Fix broken thread protection
+ * Remove SQLNumResultCols after execute, lease S4/S% to driver
+ * Fix string overrun in SQLDriverConnect
+ * Add initial support for Interix
+ *
+ * Revision 1.12  2003/03/05 09:48:44  lurcher
+ *
+ * Add some 64 bit fixes
+ *
+ * Revision 1.11  2003/02/27 12:19:39  lurcher
+ *
+ * Add the A functions as well as the W
+ *
  * Revision 1.10  2002/12/05 17:44:31  lurcher
  *
  * Display unknown return values in return logging
@@ -181,7 +203,18 @@
 
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLSetConnectAttr.c,v $ $Revision: 1.10 $";
+static char const rcsid[]= "$RCSfile: SQLSetConnectAttr.c,v $ $Revision: 1.15 $";
+
+SQLRETURN SQLSetConnectAttrA( SQLHDBC connection_handle,
+           SQLINTEGER attribute,
+           SQLPOINTER value,
+           SQLINTEGER string_length )
+{
+    return SQLSetConnectAttr( connection_handle,
+           attribute,
+           value,
+           string_length );
+}
 
 SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
            SQLINTEGER attribute,
@@ -197,7 +230,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
      */
     if ( attribute == SQL_ATTR_TRACE )
     {
-        if ((SQLINTEGER) value == SQL_OPT_TRACE_OFF )
+        if ((SQLLEN) value == SQL_OPT_TRACE_OFF )
         {
             char force_string[ 30 ];
 
@@ -275,11 +308,11 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
             \n\t\t\tConnection = %p\
             \n\t\t\tAttribute = %s\
             \n\t\t\tValue = %p\
-            \n\t\t\tStrLen = %p",
+            \n\t\t\tStrLen = %d",
                 connection,
                 __con_attr_as_string( s1, attribute ),
                 value, 
-                (void*)string_length );
+                (int)string_length );
 
         dm_log_write( __FILE__, 
                 __LINE__, 
@@ -305,9 +338,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                 ERROR_08003, NULL,
                 connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_DBC, connection );
-
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
     }
     else if ( connection -> state == STATE_C3 )
@@ -322,9 +353,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
             ERROR_HY010, NULL,
             connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
     else if ( connection -> state == STATE_C4 ||
             connection -> state == STATE_C5 ||
@@ -342,9 +371,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                 ERROR_08002, NULL,
                 connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_DBC, connection );
-
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
         else if ( attribute == SQL_ATTR_PACKET_SIZE )
         {
@@ -358,9 +385,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                 ERROR_HY011, NULL,
                 connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_DBC, connection );
-
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
     }
 
@@ -375,7 +400,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
      */
     if ( attribute == SQL_ATTR_LOGIN_TIMEOUT )
     {
-        connection -> login_timeout = ( SQLINTEGER ) value;
+        connection -> login_timeout = ( SQLLEN ) value;
         connection -> login_timeout_set = 1;
     }
 
@@ -392,41 +417,41 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
 
         if ( attribute == SQL_ATTR_ODBC_CURSORS )
         {
-            connection -> cursors = ( SQLINTEGER ) value;
+            connection -> cursors = ( SQLLEN ) value;
         }
         else if ( attribute == SQL_ATTR_ACCESS_MODE )
         {
-            connection -> access_mode = ( SQLINTEGER ) value;
+            connection -> access_mode = ( SQLLEN ) value;
             connection -> access_mode_set = 1;
         }
         else if ( attribute == SQL_ATTR_ASYNC_ENABLE )
         {
-            connection -> async_enable = ( SQLINTEGER ) value;
+            connection -> async_enable = ( SQLLEN ) value;
             connection -> async_enable_set = 1;
         }
         else if ( attribute == SQL_ATTR_AUTO_IPD )
         {
-            connection -> auto_ipd = ( SQLINTEGER ) value;
+            connection -> auto_ipd = ( SQLLEN ) value;
             connection -> auto_ipd_set = 1;
         }
         else if ( attribute == SQL_ATTR_AUTOCOMMIT )
         {
-            connection -> auto_commit = ( SQLINTEGER ) value;
+            connection -> auto_commit = ( SQLLEN ) value;
             connection -> auto_commit_set = 1;
         }
         else if ( attribute == SQL_ATTR_CONNECTION_TIMEOUT )
         {
-            connection -> connection_timeout = ( SQLINTEGER ) value;
+            connection -> connection_timeout = ( SQLLEN ) value;
             connection -> connection_timeout_set = 1;
         }
         else if ( attribute == SQL_ATTR_METADATA_ID )
         {
-            connection -> metadata_id = ( SQLINTEGER ) value;
+            connection -> metadata_id = ( SQLLEN ) value;
             connection -> metadata_id_set = 1;
         }
         else if ( attribute == SQL_ATTR_PACKET_SIZE )
         {
-            connection -> packet_size = ( SQLINTEGER ) value;
+            connection -> packet_size = ( SQLLEN ) value;
             connection -> packet_size_set = 1;
         }
         else if ( attribute == SQL_ATTR_QUIET_MODE )
@@ -436,10 +461,10 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
         }
         else if ( attribute == SQL_ATTR_TXN_ISOLATION )
         {
-            connection -> txn_isolation = ( SQLINTEGER ) value;
+            connection -> txn_isolation = ( SQLLEN ) value;
             connection -> txn_isolation_set = 1;
         }
-        else
+        else if ( attribute != SQL_ATTR_LOGIN_TIMEOUT )
         {
             /*
              * save any unknown attributes untill connect
@@ -461,7 +486,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
             }
             else
             {
-                sa -> int_attr = ( SQLINTEGER ) value;
+                sa -> int_attr = ( SQLLEN ) value;
             }
             sa -> next = connection -> save_attr;
             connection -> save_attr = sa;
@@ -477,9 +502,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                 LOG_INFO, 
                 connection -> msg );
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return SQL_SUCCESS;
+        return function_return( SQL_HANDLE_DBC, connection, SQL_SUCCESS );
     }
     else
     {
@@ -506,9 +529,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                                 ERROR_HY092, NULL,
                                 connection -> environment -> requested_version );
 
-                        thread_release( SQL_HANDLE_STMT, connection );
-
-                        return function_return( connection, SQL_ERROR );
+                        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
                     }
 
                     ret = SQLSETCONNECTOPTION( connection,
@@ -528,9 +549,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                             ERROR_IM001, NULL,
                             connection -> environment -> requested_version );
 
-                    thread_release( SQL_HANDLE_DBC, connection );
-
-                    return function_return( connection, SQL_ERROR );
+                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
                 }
             }
             else
@@ -567,9 +586,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                                 ERROR_HY092, NULL,
                                 connection -> environment -> requested_version );
 
-                        thread_release( SQL_HANDLE_STMT, connection );
-
-                        return function_return( connection, SQL_ERROR );
+                        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
                     }
 
                     switch( attribute )
@@ -609,9 +626,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                             ERROR_IM001, NULL,
                             connection -> environment -> requested_version );
 
-                    thread_release( SQL_HANDLE_DBC, connection );
-
-                    return function_return( connection, SQL_ERROR );
+                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
                 }
             }
             else
@@ -666,10 +681,8 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
 
     if ( attribute == SQL_ATTR_USE_BOOKMARKS && SQL_SUCCEEDED( ret ))
     {
-        connection -> bookmarks_on = (SQLUINTEGER) value;
+        connection -> bookmarks_on = (SQLLEN) value;
     }
 
-    thread_release( SQL_HANDLE_DBC, connection );
-
-    return function_return( connection, ret );
+    return function_return( SQL_HANDLE_DBC, connection, ret );
 }
