@@ -27,9 +27,20 @@
  *
  **********************************************************************
  *
- * $Id: SQLSetConnectAttrW.c,v 1.8 2002/12/05 17:44:31 lurcher Exp $
+ * $Id: SQLSetConnectAttrW.c,v 1.10 2003/10/30 18:20:46 lurcher Exp $
  *
  * $Log: SQLSetConnectAttrW.c,v $
+ * Revision 1.10  2003/10/30 18:20:46  lurcher
+ *
+ * Fix broken thread protection
+ * Remove SQLNumResultCols after execute, lease S4/S% to driver
+ * Fix string overrun in SQLDriverConnect
+ * Add initial support for Interix
+ *
+ * Revision 1.9  2003/03/05 09:48:44  lurcher
+ *
+ * Add some 64 bit fixes
+ *
  * Revision 1.8  2002/12/05 17:44:31  lurcher
  *
  * Display unknown return values in return logging
@@ -118,7 +129,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
 
     if ( attribute == SQL_ATTR_TRACE )
     {
-        if ((SQLINTEGER) value == SQL_OPT_TRACE_OFF )
+        if ((SQLLEN) value == SQL_OPT_TRACE_OFF )
         {
             char force_string[ 30 ];
 
@@ -226,9 +237,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                 ERROR_08003, NULL,
                 connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_DBC, connection );
-
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
     }
     else if ( connection -> state == STATE_C3 )
@@ -243,9 +252,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
             ERROR_HY010, NULL,
             connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, SQL_ERROR );
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
     else if ( connection -> state == STATE_C4 ||
             connection -> state == STATE_C5 || 
@@ -263,9 +270,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                 ERROR_08002, NULL,
                 connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_DBC, connection );
-
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
         else if ( attribute == SQL_ATTR_PACKET_SIZE )
         {
@@ -279,9 +284,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                 ERROR_HY011, NULL,
                 connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_DBC, connection );
-
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
     }
 
@@ -296,7 +299,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
      */
     if ( attribute == SQL_ATTR_LOGIN_TIMEOUT )
     {
-        connection -> login_timeout = ( SQLINTEGER ) value;
+        connection -> login_timeout = ( SQLLEN ) value;
         connection -> login_timeout_set = 1;
     }
 
@@ -313,41 +316,41 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
 
         if ( attribute == SQL_ATTR_ODBC_CURSORS )
         {
-            connection -> cursors = ( SQLINTEGER ) value;
+            connection -> cursors = ( SQLLEN ) value;
         }
         else if ( attribute == SQL_ATTR_ACCESS_MODE )
         {
-            connection -> access_mode = ( SQLINTEGER ) value;
+            connection -> access_mode = ( SQLLEN ) value;
             connection -> access_mode_set = 1;
         }
         else if ( attribute == SQL_ATTR_ASYNC_ENABLE )
         {
-            connection -> async_enable = ( SQLINTEGER ) value;
+            connection -> async_enable = ( SQLLEN ) value;
             connection -> async_enable_set = 1;
         }
         else if ( attribute == SQL_ATTR_AUTO_IPD )
         {
-            connection -> auto_ipd = ( SQLINTEGER ) value;
+            connection -> auto_ipd = ( SQLLEN ) value;
             connection -> auto_ipd_set = 1;
         }
         else if ( attribute == SQL_ATTR_AUTOCOMMIT )
         {
-            connection -> auto_commit = ( SQLINTEGER ) value;
+            connection -> auto_commit = ( SQLLEN ) value;
             connection -> auto_commit_set = 1;
         }
         else if ( attribute == SQL_ATTR_CONNECTION_TIMEOUT )
         {
-            connection -> connection_timeout = ( SQLINTEGER ) value;
+            connection -> connection_timeout = ( SQLLEN ) value;
             connection -> connection_timeout_set = 1;
         }
         else if ( attribute == SQL_ATTR_METADATA_ID )
         {
-            connection -> metadata_id = ( SQLINTEGER ) value;
+            connection -> metadata_id = ( SQLLEN ) value;
             connection -> metadata_id_set = 1;
         }
         else if ( attribute == SQL_ATTR_PACKET_SIZE )
         {
-            connection -> packet_size = ( SQLINTEGER ) value;
+            connection -> packet_size = ( SQLLEN ) value;
             connection -> packet_size_set = 1;
         }
         else if ( attribute == SQL_ATTR_QUIET_MODE )
@@ -357,7 +360,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
         }
         else if ( attribute == SQL_ATTR_TXN_ISOLATION )
         {
-            connection -> txn_isolation = ( SQLINTEGER ) value;
+            connection -> txn_isolation = ( SQLLEN ) value;
             connection -> txn_isolation_set = 1;
         }
         else
@@ -382,7 +385,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
             }
             else
             {
-                sa -> int_attr = ( SQLINTEGER ) value;
+                sa -> int_attr = ( SQLLEN ) value;
             }
             sa -> next = connection -> save_attr;
             connection -> save_attr = sa;
@@ -398,9 +401,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                 LOG_INFO, 
                 connection -> msg );
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return SQL_SUCCESS;
+        return function_return( SQL_HANDLE_DBC, connection, SQL_SUCCESS );
     }
     else
     {
@@ -427,9 +428,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                                 ERROR_HY092, NULL,
                                 connection -> environment -> requested_version );
 
-                        thread_release( SQL_HANDLE_STMT, connection );
-
-                        return function_return( connection, SQL_ERROR );
+                        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
                     }
 
                     ret = SQLSETCONNECTOPTIONW( connection,
@@ -449,10 +448,8 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                             ERROR_IM001, NULL,
                             connection -> environment -> requested_version );
 
-                    thread_release( SQL_HANDLE_DBC, connection );
-
-                    return function_return( connection, SQL_ERROR );
-                    }
+                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                }
             }
             else
             {
@@ -488,9 +485,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                                 ERROR_HY092, NULL,
                                 connection -> environment -> requested_version );
 
-                        thread_release( SQL_HANDLE_STMT, connection );
-
-                        return function_return( connection, SQL_ERROR );
+                        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
                     }
 
                     switch( attribute )
@@ -524,10 +519,8 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                             ERROR_IM001, NULL,
                             connection -> environment -> requested_version );
 
-                    thread_release( SQL_HANDLE_DBC, connection );
-
-                    return function_return( connection, SQL_ERROR );
-                    }
+                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                }
             }
             else
             {
@@ -583,10 +576,8 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
 
     if ( attribute == SQL_ATTR_USE_BOOKMARKS && SQL_SUCCEEDED( ret ))
     {
-        connection -> bookmarks_on = (SQLUINTEGER) value;
+        connection -> bookmarks_on = (SQLULEN) value;
     }
 
-    thread_release( SQL_HANDLE_DBC, connection );
-
-    return function_return( connection, ret );
+    return function_return( SQL_HANDLE_DBC, connection, ret );
 }
