@@ -27,9 +27,24 @@
  *
  **********************************************************************
  *
- * $Id: SQLSetStmtAttr.c,v 1.6 2002/12/05 17:44:31 lurcher Exp $
+ * $Id: SQLSetStmtAttr.c,v 1.9 2003/10/30 18:20:46 lurcher Exp $
  *
  * $Log: SQLSetStmtAttr.c,v $
+ * Revision 1.9  2003/10/30 18:20:46  lurcher
+ *
+ * Fix broken thread protection
+ * Remove SQLNumResultCols after execute, lease S4/S% to driver
+ * Fix string overrun in SQLDriverConnect
+ * Add initial support for Interix
+ *
+ * Revision 1.8  2003/03/05 09:48:45  lurcher
+ *
+ * Add some 64 bit fixes
+ *
+ * Revision 1.7  2003/02/27 12:19:40  lurcher
+ *
+ * Add the A functions as well as the W
+ *
  * Revision 1.6  2002/12/05 17:44:31  lurcher
  *
  * Display unknown return values in return logging
@@ -173,7 +188,18 @@
 
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLSetStmtAttr.c,v $ $Revision: 1.6 $";
+static char const rcsid[]= "$RCSfile: SQLSetStmtAttr.c,v $ $Revision: 1.9 $";
+
+SQLRETURN SQLSetStmtAttrA( SQLHSTMT statement_handle,
+           SQLINTEGER attribute,
+           SQLPOINTER value,
+           SQLINTEGER string_length )
+{
+    return SQLSetStmtAttr( statement_handle,
+           attribute,
+           value,
+           string_length );
+}
 
 SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
            SQLINTEGER attribute,
@@ -246,9 +272,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                  ERROR_HY011, NULL,
                  statement -> connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_STMT, statement );
-
-            return function_return( statement, SQL_ERROR );
+            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
         else if ( statement -> state == STATE_S4 ||
                 statement -> state == STATE_S5 ||
@@ -265,9 +289,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                  ERROR_24000, NULL,
                  statement -> connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_STMT, statement );
-
-            return function_return( statement, SQL_ERROR );
+            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
         else if ( statement -> state == STATE_S8 ||
                 statement -> state == STATE_S9 ||
@@ -287,9 +309,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                      ERROR_HY011, NULL,
                      statement -> connection -> environment -> requested_version );
 
-                thread_release( SQL_HANDLE_STMT, statement );
-
-                return function_return( statement, SQL_ERROR );
+                return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
             }
             else
             {
@@ -303,9 +323,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                      ERROR_HY010, NULL,
                      statement -> connection -> environment -> requested_version );
 
-                thread_release( SQL_HANDLE_STMT, statement );
-
-                return function_return( statement, SQL_ERROR );
+                return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
             }
         }
     }
@@ -327,9 +345,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                  ERROR_HY010, NULL,
                  statement -> connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_STMT, statement );
-
-            return function_return( statement, SQL_ERROR );
+            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
     }
 
@@ -346,9 +362,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                 ERROR_IM001, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        thread_release( SQL_HANDLE_STMT, statement );
-
-        return function_return( statement, SQL_ERROR );
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     /*
@@ -389,9 +403,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                     ERROR_HY017, NULL,
                     statement -> connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_STMT, statement );
-
-            return function_return( statement, SQL_ERROR );
+            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
 
         if ( desc -> connection !=
@@ -407,9 +419,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                     ERROR_HY024, NULL,
                     statement -> connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_STMT, statement );
-
-            return function_return( statement, SQL_ERROR );
+            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
 
         /*
@@ -453,9 +463,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                     ERROR_HY017, NULL,
                     statement -> connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_STMT, statement );
-
-            return function_return( statement, SQL_ERROR );
+            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
 
         if ( desc -> connection !=
@@ -471,9 +479,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                     ERROR_HY024, NULL,
                     statement -> connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_STMT, statement );
-
-            return function_return( statement, SQL_ERROR );
+            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
 
         /*
@@ -489,7 +495,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
 
     if ( attribute == SQL_ATTR_METADATA_ID )
     {
-        statement -> metadata_id = (int) value;
+        statement -> metadata_id = (SQLLEN) value;
     }
 
     /*
@@ -591,7 +597,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
         /*
          * save this incase we need it in SQLExtendedFetch
          */
-        statement -> row_array_size = (SQLUINTEGER) value;
+        statement -> row_array_size = (SQLULEN) value;
 
         if ( CHECK_SQLSETSTMTATTR( statement -> connection ))
         {
@@ -636,9 +642,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                     ERROR_HY092, NULL,
                     statement -> connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_STMT, statement );
-
-            return function_return( statement, SQL_ERROR );
+            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
 
         ret = SQLSETSTMTOPTION( statement -> connection,
@@ -653,7 +657,7 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
 
     if ( attribute == SQL_ATTR_USE_BOOKMARKS && SQL_SUCCEEDED( ret ))
     {
-        statement -> bookmarks_on = (SQLUINTEGER) value;
+        statement -> bookmarks_on = (SQLULEN) value;
     }
 
     if ( log_info.log_flag )
@@ -669,7 +673,5 @@ SQLRETURN SQLSetStmtAttr( SQLHSTMT statement_handle,
                 statement -> msg );
     }
 
-    thread_release( SQL_HANDLE_STMT, statement );
-
-    return function_return( statement, ret ); 
+    return function_return( SQL_HANDLE_STMT, statement, ret ); 
 }
