@@ -27,9 +27,21 @@
  *
  **********************************************************************
  *
- * $Id: SQLTransact.c,v 1.6 2002/12/05 17:44:31 lurcher Exp $
+ * $Id: SQLTransact.c,v 1.8 2004/06/16 14:42:03 lurcher Exp $
  *
  * $Log: SQLTransact.c,v $
+ * Revision 1.8  2004/06/16 14:42:03  lurcher
+ *
+ *
+ * Fix potential corruption with threaded use and SQLEndTran
+ *
+ * Revision 1.7  2003/10/30 18:20:46  lurcher
+ *
+ * Fix broken thread protection
+ * Remove SQLNumResultCols after execute, lease S4/S% to driver
+ * Fix string overrun in SQLDriverConnect
+ * Add initial support for Interix
+ *
  * Revision 1.6  2002/12/05 17:44:31  lurcher
  *
  * Display unknown return values in return logging
@@ -123,7 +135,7 @@
 
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLTransact.c,v $ $Revision: 1.6 $";
+static char const rcsid[]= "$RCSfile: SQLTransact.c,v $ $Revision: 1.8 $";
 
 SQLRETURN SQLTransact( SQLHENV environment_handle,
            SQLHDBC connection_handle,
@@ -182,9 +194,7 @@ SQLRETURN SQLTransact( SQLHENV environment_handle,
                     ERROR_08003, NULL,
                     connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_DBC, connection );
-
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
 
         if ( completion_type != SQL_COMMIT &&
@@ -200,9 +210,7 @@ SQLRETURN SQLTransact( SQLHENV environment_handle,
                     ERROR_HY012, NULL,
                     connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_DBC, connection );
-
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
 
         if ( CHECK_SQLTRANSACT( connection ))
@@ -231,9 +239,7 @@ SQLRETURN SQLTransact( SQLHENV environment_handle,
                     ERROR_IM001, NULL,
                     connection -> environment -> requested_version );
 
-            thread_release( SQL_HANDLE_DBC, connection );
-
-            return function_return( connection, SQL_ERROR );
+            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
 
         if( SQL_SUCCEEDED(ret) )
@@ -249,9 +255,6 @@ SQLRETURN SQLTransact( SQLHENV environment_handle,
              * relative to the commit or rollback behavior
              */
             
-            statement = __get_stmt_root();
-            stmt_remaining = connection -> statement_count;
-            
             /* release thread so we can get the info */
             thread_release( SQL_HANDLE_DBC, connection );
             
@@ -274,6 +277,16 @@ SQLRETURN SQLTransact( SQLHENV environment_handle,
 	    
             /* protect thread again */
             thread_protect( SQL_HANDLE_DBC, connection );
+
+	    /* 
+	     * We need to protect this at a higher level than connection
+	     * as statements can be coming and going
+	     */
+
+	    mutex_lib_entry();
+
+            statement = __get_stmt_root();
+            stmt_remaining = connection -> statement_count;
 	    
             if( SQL_SUCCEEDED( ret1 ) )
             {
@@ -320,6 +333,8 @@ SQLRETURN SQLTransact( SQLHENV environment_handle,
                     statement = statement -> next_class_list;
                 }
             }
+
+	    mutex_lib_exit();
         }
 
         if ( log_info.log_flag )
@@ -335,9 +350,7 @@ SQLRETURN SQLTransact( SQLHENV environment_handle,
                     connection -> msg );
         }
 
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return function_return( connection, ret );
+        return function_return( SQL_HANDLE_DBC, connection, ret );
     }
     else if ( environment_handle != SQL_NULL_HENV )
     {
@@ -389,9 +402,7 @@ SQLRETURN SQLTransact( SQLHENV environment_handle,
                     ERROR_08003, NULL,
                     environment -> requested_version );
 
-            thread_release( SQL_HANDLE_ENV, environment );
-
-            return function_return( environment, SQL_ERROR );
+            return function_return( SQL_HANDLE_ENV, environment, SQL_ERROR );
         }
 
         if ( completion_type != SQL_COMMIT &&
@@ -409,7 +420,7 @@ SQLRETURN SQLTransact( SQLHENV environment_handle,
 
             thread_release( SQL_HANDLE_ENV, environment );
 
-            return function_return( environment, SQL_ERROR );
+            return function_return( SQL_HANDLE_ENV, environment, SQL_ERROR );
         }
 
         /*
@@ -444,7 +455,7 @@ SQLRETURN SQLTransact( SQLHENV environment_handle,
 
                         thread_release( SQL_HANDLE_ENV, environment );
 
-                        return function_return( environment, SQL_ERROR );
+                        return function_return( SQL_HANDLE_ENV, environment, SQL_ERROR );
                     }
                 }
                 else if ( CHECK_SQLENDTRAN( connection ))
@@ -466,9 +477,7 @@ SQLRETURN SQLTransact( SQLHENV environment_handle,
                                 ERROR_25S01, NULL,
                                 environment -> requested_version );
 
-                        thread_release( SQL_HANDLE_ENV, environment );
-
-                        return function_return( environment, SQL_ERROR );
+                        return function_return( SQL_HANDLE_ENV, environment, SQL_ERROR );
                     }
                 }
                 else
@@ -483,9 +492,7 @@ SQLRETURN SQLTransact( SQLHENV environment_handle,
                         ERROR_IM001, NULL,
                         environment -> requested_version );
 
-                    thread_release( SQL_HANDLE_ENV, environment );
-
-                    return function_return( environment, SQL_ERROR );
+                    return function_return( SQL_HANDLE_ENV, environment, SQL_ERROR );
                 }
             }
 
