@@ -27,9 +27,23 @@
  *
  **********************************************************************
  *
- * $Id: __handles.c,v 1.4 2002/08/12 16:20:44 lurcher Exp $
+ * $Id: __handles.c,v 1.8 2004/09/28 08:44:46 lurcher Exp $
  *
  * $Log: __handles.c,v $
+ * Revision 1.8  2004/09/28 08:44:46  lurcher
+ * Fix memory leak in pthread descriptor code
+ *
+ * Revision 1.7  2004/07/24 17:55:37  lurcher
+ * Sync up CVS
+ *
+ * Revision 1.6  2003/06/04 12:49:45  lurcher
+ *
+ * Further PID logging tweeks
+ *
+ * Revision 1.5  2003/06/02 16:51:36  lurcher
+ *
+ * Add TracePid option
+ *
  * Revision 1.4  2002/08/12 16:20:44  lurcher
  *
  * Make it try and find a working iconv set of encodings
@@ -196,7 +210,7 @@
 #include <uodbc_stats.h>
 #endif
 
-static char const rcsid[]= "$RCSfile: __handles.c,v $ $Revision: 1.4 $";
+static char const rcsid[]= "$RCSfile: __handles.c,v $ $Revision: 1.8 $";
 
 /*
  * these are used to enable us to check if a handle is
@@ -250,6 +264,7 @@ static DMHDESC descriptor_root;
 static pth_mutex_t mutex_lists = PTH_MUTEX_INIT;
 static pth_mutex_t mutex_env = PTH_MUTEX_INIT;
 static pth_mutex_t mutex_pool = PTH_MUTEX_INIT;
+static pth_mutex_t mutex_iconv = PTH_MUTEX_INIT;
 static int pth_init_called = 0;
 
 static int mutex_entry( pth_mutex_t *mutex )
@@ -274,6 +289,7 @@ static int mutex_exit( pth_mutex_t *mutex )
 static pthread_mutex_t mutex_lists = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t mutex_env = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t mutex_pool = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t mutex_iconv = PTHREAD_MUTEX_INITIALIZER;
 
 static int mutex_entry( pthread_mutex_t *mutex )
 {
@@ -292,6 +308,7 @@ static int mutex_exit( pthread_mutex_t *mutex )
 static mutex_t mutex_lists;
 static mutex_t mutex_env;
 static mutex_t mutex_pool;
+static mutex_t mutex_iconv;
 
 static int mutex_entry( mutex_t *mutex )
 {
@@ -324,6 +341,20 @@ void mutex_pool_exit( void )
     mutex_exit( &mutex_pool );
 }
 
+/*
+ * protection for iconv
+ */
+
+void mutex_iconv_entry( void )
+{
+    mutex_entry( &mutex_iconv );
+}
+
+void mutex_iconv_exit( void )
+{
+    mutex_exit( &mutex_iconv );
+}
+
 /*
  * protection for lib loading and counting, reuse the lists mutex as this 
  * is the lowest level protection the DM uses
@@ -390,7 +421,21 @@ DMHENV __alloc_env( void )
              * start logging
              */
 
-            dm_log_open( "ODBC", tracing_file );
+            SQLGetPrivateProfileString( "ODBC", "TracePid", "No",
+                    tracing_string, sizeof( tracing_string ), 
+                    "odbcinst.ini" );
+
+            if ( tracing_string[ 0 ] == '1' ||
+                        toupper( tracing_string[ 0 ] ) == 'Y' ||
+                        ( toupper( tracing_string[ 0 ] ) == 'O' &&
+                        toupper( tracing_string[ 1 ] ) == 'N' ))
+            {
+                dm_log_open( "ODBC", tracing_file, 1 );
+            }
+            else
+            {
+                dm_log_open( "ODBC", tracing_file, 0 );
+            }
 
             sprintf( environment -> msg,
                     "\n\t\tExit:[SQL_SUCCESS]\n\t\t\tEnvironment = %p",	environment );
@@ -1070,7 +1115,7 @@ void __release_desc( DMHDESC descriptor )
 
     clear_error_head( &descriptor -> error );
 
-#ifdef HAVE_LIBPTHREAD
+#ifdef HAVE_LIBPTH
 #elif HAVE_LIBPTHREAD
     pthread_mutex_destroy( &descriptor -> mutex );
 #elif HAVE_LIBTHREAD
