@@ -168,7 +168,7 @@ typedef struct error_head
     EHEADER     sql_diag_head;
     void        *owning_handle;
     int         handle_type;
-    SQLINTEGER  return_code;
+    SQLRETURN   return_code;
     SQLINTEGER  header_set;
     SQLRETURN   diag_cursor_row_count_ret;
     SQLRETURN   diag_dynamic_function_ret;
@@ -187,6 +187,9 @@ struct log_structure
     char    *program_name;
     char    *log_file_name;
     int     log_flag;
+    int     pid_logging;            /* the log path specifies a directory, and a */
+                                    /* log file per pid is created */
+
 };
 
 extern struct log_structure log_info;
@@ -341,16 +344,17 @@ typedef struct connection
     char            password[ 128 ];
     int             password_length;
     char            cli_year[ 5 ];
-    struct attr_struct  env_attribute;  /* Extended attribute set info */
+    struct attr_struct  env_attribute;      /* Extended attribute set info */
     struct attr_struct  dbc_attribute;
     struct attr_struct  stmt_attribute;
-    struct save_attr    *save_attr;     /* SQLConnectAttr before connect */
+    struct save_attr    *save_attr;         /* SQLConnectAttr before connect */
 #ifdef HAVE_ICONV
-    iconv_t         iconv_cd_uc_to_ascii;  /* in and out conversion descriptor */
+    iconv_t         iconv_cd_uc_to_ascii;   /* in and out conversion descriptor */
     iconv_t         iconv_cd_ascii_to_uc;
-    char            unicode_string[ 64 ];  /* name of unicode conversion */
+    char            unicode_string[ 64 ];   /* name of unicode conversion */
 #endif
-    struct env_lib_struct *env_list_ent;/* pointer to reference in the env list */
+    struct env_lib_struct *env_list_ent;    /* pointer to reference in the env list */
+    char            probe_sql[ 512 ];       /* SQL to use to check a pool is valid */
 } *DMHDBC;
 
 typedef struct connection_pool
@@ -399,7 +403,7 @@ typedef struct statement
     int             state;              /* state of statement */
     DMHDBC          connection;         /* DM connection that owns this */
     SQLHANDLE       driver_stmt;        /* statement in the driver */
-    SQLSMALLINT     numcols;            /* Number of columns in the result set */
+    SQLSMALLINT     hascols;            /* is there a result set */
     int             prepared;           /* the statement has been prepared */
     int             interupted_func;    /* current function running async */
                                         /* or NEED_DATA */
@@ -443,6 +447,9 @@ void mutex_lib_exit( void );
 void mutex_pool_entry( void );
 void mutex_pool_exit( void );
 
+void mutex_iconv_entry( void );
+void mutex_iconv_exit( void );
+
 typedef struct connection_pair
 {
     char            *name;
@@ -493,6 +500,7 @@ void __disconnect_part_one( DMHDBC connection );
 int __connect_part_two( DMHDBC connection );
 void __disconnect_part_two( DMHDBC connection );
 void __disconnect_part_three( DMHDBC connection );
+void __disconnect_part_four( DMHDBC connection );
 DMHDBC __get_dbc_root( void );
 DMHSTMT __get_stmt_root( void );
 
@@ -534,6 +542,7 @@ typedef enum error_id
     ERROR_25000,
     ERROR_25S01,
     ERROR_S1000,
+    ERROR_S1003,
     ERROR_S1010,
     ERROR_S1011,
     ERROR_S1107,
@@ -559,6 +568,7 @@ typedef enum error_id
     ERROR_HY105,
     ERROR_HY106,
     ERROR_HY110,
+    ERROR_HY111,
     ERROR_HYC00,
     ERROR_IM001,
     ERROR_IM002,
@@ -572,7 +582,9 @@ typedef enum error_id
     ERROR_SL010
 } error_id;
 
-#define function_return(h,r)    function_return_ex(h,r,FALSE)
+#define IGNORE_THREAD       (-1)
+
+#define function_return(l,h,r)    function_return_ex(l,h,r,FALSE)
 
 #define SUBCLASS_ODBC           0
 #define SUBCLASS_ISO            1
@@ -593,7 +605,7 @@ void __post_internal_error_ex_w( EHEAD *error_handle,
         SQLWCHAR *message_text,
         int class_origin,
         int subclass_origin );
-int function_return_ex( void * handle, int ret_code, int save_to_diag );
+int function_return_ex( int level, void * handle, int ret_code, int save_to_diag );
 void function_entry( void *handle );
 void setup_error_head( EHEAD *error_header, void *handle, int handle_type );
 void clear_error_head( EHEAD *error_header );
@@ -686,7 +698,7 @@ char *__find_lib_name( char *dsn, char *lib_name, char *driver_name );
  * setup the cursor library
  */
 
-SQLRETURN CLConnect( DMHDBC connection, struct driver_helper_funcs * );
+SQLRETURN SQL_API CLConnect( DMHDBC connection, struct driver_helper_funcs * );
 
 /*
  * connection string functions
@@ -735,7 +747,7 @@ SQLINTEGER map_ca_odbc2_to_3( SQLINTEGER field_identifier );
  * driver manager logging functions
  */
 
-void dm_log_open( char *program_name, char *log_file );
+void dm_log_open( char *program_name, char *log_file, int pid_logging );
 
 void dm_log_write( char *function_name, int line, int type, int severity, char *message );
 void dm_log_write_diag( char *message );
