@@ -34,8 +34,10 @@
 #ifdef HAVE_CRYPT_H
 #include <crypt.h>
 #endif
+#include "md5.h"
 #endif
 
+
 #define STMT_INCREMENT 16  /* how many statement holders to allocate at a time */
 
 #define PRN_NULLCHECK
@@ -43,8 +45,7 @@
 extern GLOBAL_VALUES globals;
 
 
-RETCODE SQL_API SQLAllocConnect(
-                                HENV     henv,
+RETCODE SQL_API SQLAllocConnect(HENV     henv,
                                 HDBC FAR *phdbc)
 {
 EnvironmentClass *env = (EnvironmentClass *)henv;
@@ -57,23 +58,23 @@ static char* const func="SQLAllocConnect";
 	mylog("**** %s: henv = %u, conn = %u\n", func, henv, conn);
 
     if( ! conn) {
-        env->errormsg = "Couldn't allocate memory for Connection object.";
-        env->errornumber = ENV_ALLOC_ERROR;
-		*phdbc = SQL_NULL_HDBC;
-		EN_log_error(func, "", env);
-        return SQL_ERROR;
+	env->errornumber = ENV_ALLOC_ERROR;
+	env->errormsg = "Couldn't allocate memory for Connection object.";
+	*phdbc = SQL_NULL_HDBC;
+	EN_log_error(func, "", env);
+	return SQL_ERROR;
     }
 
     if ( ! EN_add_connection(env, conn)) {
-        env->errormsg = "Maximum number of connections exceeded.";
         env->errornumber = ENV_ALLOC_ERROR;
+        env->errormsg = "Maximum number of connections exceeded.";
         CC_Destructor(conn);
-		*phdbc = SQL_NULL_HDBC;
-		EN_log_error(func, "", env);
+	*phdbc = SQL_NULL_HDBC;
+	EN_log_error(func, "", env);
         return SQL_ERROR;
     }
 
-	*phdbc = (HDBC) conn;
+    *phdbc = (HDBC) conn;
 
     return SQL_SUCCESS;
 }
@@ -121,7 +122,7 @@ static char* const func = "SQLConnect";
 
 	qlog("conn = %u, %s(DSN='%s', UID='%s', PWD='%s')\n", conn, func, ci->dsn, ci->username, ci->password);
 
-	if ( CC_connect(conn, FALSE) <= 0) {
+	if ( CC_connect(conn, AUTH_REQ_OK, NULL) <= 0) {
 		/*	Error messages are filled in */
 		CC_log_error(func, "Error on CC_connect", conn);
 		return SQL_ERROR;
@@ -158,7 +159,6 @@ RETCODE SQL_API SQLDisconnect(
 ConnectionClass *conn = (ConnectionClass *) hdbc;
 static char* const func = "SQLDisconnect";
 
-
 	mylog( "%s: entering...\n", func);
 
 	if ( ! conn) {
@@ -169,8 +169,7 @@ static char* const func = "SQLDisconnect";
 	qlog("conn=%u, %s\n", conn, func);
 
 	if (conn->status == CONN_EXECUTING) {
-		conn->errornumber = CONN_IN_USE;
-		conn->errormsg = "A transaction is currently being executed";
+		CC_set_error(conn, CONN_IN_USE, "A transaction is currently being executed");
 		CC_log_error(func, "", conn);
 		return SQL_ERROR;
 	}
@@ -205,8 +204,7 @@ static char* const func = "SQLFreeConnect";
 
 	/*  Remove the connection from the environment */
 	if ( ! EN_remove_connection((EnvironmentClass*)conn->henv, conn)) {
-		conn->errornumber = CONN_IN_USE;
-		conn->errormsg = "A transaction is currently being executed";
+		CC_set_error(conn, CONN_IN_USE, "A transaction is currently being executed");
 		CC_log_error(func, "", conn);
 		return SQL_ERROR;
 	}
@@ -235,9 +233,7 @@ ConnectionClass *rv;
 
 		rv->henv = (HENV)NULL; /* not yet associated with an environment */
 
-        rv->errormsg = NULL;
-        rv->errornumber = 0;
-		rv->errormsg_created = FALSE;
+        CC_clear_error(rv);
 
         rv->status = CONN_NOT_CONNECTED;
         rv->transact_status = CONN_IN_AUTOCOMMIT; /* autocommit by default */
@@ -313,7 +309,6 @@ CC_Destructor(ConnectionClass *self)
 		free(self->col_info);
 	}
 
-
 	free(self);
 
 	mylog("exit CC_Destructor\n");
@@ -344,8 +339,8 @@ int i, count = 0;
 void 
 CC_clear_error(ConnectionClass *self)
 {
-	self->errornumber = 0; 
-	self->errormsg = NULL; 
+	self->__error_number = 0; 
+	self->__error_message = NULL; 
 	self->errormsg_created = FALSE;
 }
 
@@ -414,15 +409,18 @@ StatementClass *stmt;
 
 	/*  Free all the stmts on this connection */
 	for (i = 0; i < self->num_stmts; i++) {
-		stmt = self->stmts[i];
-		if (stmt) {
+        if ( self->stmts )
+        {
+            stmt = self->stmts[i];
+            if (stmt) {
 
-			stmt->hdbc = NULL;	/* prevent any more dbase interactions */
+                stmt->hdbc = NULL;	/* prevent any more dbase interactions */
 
-			SC_Destructor(stmt);
+                SC_Destructor(stmt);
 
-			self->stmts[i] = NULL;
-		}
+                self->stmts[i] = NULL;
+            }
+        }
 	}
 
 	/*	Check for translation dll */
@@ -455,8 +453,7 @@ CC_set_translation (ConnectionClass *self)
 	self->translation_handle = LoadLibrary (self->connInfo.translation_dll);
 
 	if (self->translation_handle == NULL) {
-		self->errornumber = CONN_UNABLE_TO_LOAD_DLL;
-		self->errormsg = "Could not load the translation DLL.";
+		CC_set_error(self, CONN_UNABLE_TO_LOAD_DLL, "Could not load the translation DLL.");
 		return FALSE;
 	}
 
@@ -469,31 +466,64 @@ CC_set_translation (ConnectionClass *self)
 												"SQLDriverToDataSource");
 
 	if (self->DataSourceToDriver == NULL || self->DriverToDataSource == NULL) {
-		self->errornumber = CONN_UNABLE_TO_LOAD_DLL;
-		self->errormsg = "Could not find translation DLL functions.";
+		CC_set_error(self, CONN_UNABLE_TO_LOAD_DLL, "Could not find translation DLL functions.");
 		return FALSE;
 	}
 #endif
 	return TRUE;
 }
 
-char 
-CC_connect(ConnectionClass *self, char do_password)
+static	int
+md5_auth_send(ConnectionClass *self, const char *salt)
 {
-StartupPacket sp;
-StartupPacket6_2 sp62;
-QResultClass *res;
-SocketClass *sock;
-ConnInfo *ci = &(self->connInfo);
-int areq = -1;
-int beresp;
-char msgbuffer[ERROR_MSG_LENGTH]; 
-char salt[2];
-static char* const func="CC_connect";
+	char	*pwd1 = NULL, *pwd2 = NULL;
+	ConnInfo   *ci = &(self->connInfo);
+	SocketClass	*sock = self->sock;
 
+	if (!(pwd1 = malloc(MD5_PASSWD_LEN + 1)))
+		return 1;
+	if (!EncryptMD5(ci->password, ci->username, strlen(ci->username), pwd1))
+	{
+		free(pwd1);
+		return 1;
+	} 
+	if (!(pwd2 = malloc(MD5_PASSWD_LEN + 1)))
+	{
+		free(pwd1);
+		return 1;
+	} 
+	if (!EncryptMD5(pwd1 + strlen("md5"), salt, 4, pwd2))
+	{
+		free(pwd2);
+		free(pwd1);
+		return 1;
+	}
+	free(pwd1);
+	SOCK_put_int(sock, 4 + strlen(pwd2) + 1, 4);
+	SOCK_put_n_char(sock, pwd2, strlen(pwd2) + 1);
+	SOCK_flush_output(sock);
+	free(pwd2);
+	return 0; 
+}
+
+char 
+CC_connect(ConnectionClass *self, char password_req, char *salt_para)
+{
+	StartupPacket	sp;
+	StartupPacket6_2 sp62;
+	QResultClass 	*res;
+	SocketClass	 *sock;
+	ConnInfo 	*ci = &(self->connInfo);
+	int 		areq = -1;
+	int 		beresp;
+	char 		msgbuffer[ERROR_MSG_LENGTH]; 
+	char 		salt[5], notice[512];
+	static char* const func="CC_connect";
+	char		*encoding;
+	
 	mylog("%s: entering...\n", func);
 
-	if ( do_password)
+	if (password_req != AUTH_REQ_OK)
 
 		sock = self->sock;		/* already connected, just authenticate */
 
@@ -520,27 +550,26 @@ static char* const func="CC_connect";
 			globals.conn_settings);
 
 		if (self->status != CONN_NOT_CONNECTED) {
-			self->errormsg = "Already connected.";
-			self->errornumber = CONN_OPENDB_ERROR;
+			CC_set_error(self, CONN_OPENDB_ERROR, "Already connected.");
 			return 0;
 		}
 
 		if ( ci->server[0] == '\0' || ci->port[0] == '\0' || ci->database[0] == '\0') {
-			self->errornumber = CONN_INIREAD_ERROR;
-			self->errormsg = "Missing server name, port, or database name in call to CC_connect.";
+			CC_set_error(self, CONN_INIREAD_ERROR,"Missing server name, port, or database name in call to CC_connect.");
 			return 0;
 		}
 
 		mylog("CC_connect(): DSN = '%s', server = '%s', port = '%s', database = '%s', username = '%s', password='%s'\n", ci->dsn, ci->server, ci->port, ci->database, ci->username, ci->password);
 
+another_version_retry:
+
 		/* If the socket was closed for some reason (like a SQLDisconnect, but no SQLFreeConnect
-			then create a socket now.
-		*/
+		 * then create a socket now.
+		 */
 		if ( ! self->sock) {
 			self->sock = SOCK_Constructor();
 			if ( ! self->sock) {
-				 self->errornumber = CONNECTION_SERVER_NOT_REACHED;
-				 self->errormsg = "Could not open a socket to the server";
+				 CC_set_error(self, CONNECTION_SERVER_NOT_REACHED, "Could not open a socket to the server");
 				 return 0;
 			}
 		}
@@ -552,8 +581,7 @@ static char* const func="CC_connect";
 		SOCK_connect_to(sock, (short) atoi(ci->port), ci->server, ci->uds);
 		if (SOCK_get_errcode(sock) != 0) {
 			mylog("connection to the server socket failed.\n");
-			self->errornumber = CONNECTION_SERVER_NOT_REACHED;
-			self->errormsg = "Could not connect to the server";
+			CC_set_error(self, CONNECTION_SERVER_NOT_REACHED, "Could not connect to the server");
 			return 0;
 		}
 		mylog("connection to the server socket succeeded.\n");
@@ -593,8 +621,7 @@ static char* const func="CC_connect";
 
 		if (sock->errornumber != 0) {
 			mylog("couldn't send the authentication block properly.\n");
-			self->errornumber = CONN_INVALID_AUTHENTICATION;
-			self->errormsg = "Sending the authentication packet failed";
+			CC_set_error(self, CONN_INVALID_AUTHENTICATION, "Sending the authentication packet failed");
 			return 0;
 		}
 		mylog("sent the authentication block successfully.\n");
@@ -608,113 +635,168 @@ static char* const func="CC_connect";
 	/*	Now get the authentication request from backend */
 	/* *************************************************** */
 
-	if ( ! PROTOCOL_62(ci))	do {
+	if (!PROTOCOL_62(ci))
+	{
+		BOOL		before_64 = PG_VERSION_LT(self, 6.4),
+					ReadyForQuery = FALSE;
 
-		if (do_password)
-			beresp = 'R';
-		else
-			beresp = SOCK_get_char(sock);
-
-		switch(beresp) {
-		case 'E':
-			mylog("auth got 'E'\n");
-
-			SOCK_get_string(sock, msgbuffer, ERROR_MSG_LENGTH);
-			self->errornumber = CONN_INVALID_AUTHENTICATION;
-			self->errormsg = msgbuffer;
-			qlog("ERROR from backend during authentication: '%s'\n", self->errormsg);
-			return 0;
-		case 'R':
-
-			if (do_password) {
-				mylog("in 'R' do_password\n");
-				areq = AUTH_REQ_PASSWORD;
-				do_password = FALSE;
+		do
+		{
+			if (password_req != AUTH_REQ_OK)
+				beresp = 'R';
+			else
+			{
+				beresp = SOCK_get_char(sock);
+				mylog("auth got '%c'\n", beresp);
 			}
-			else {
-				mylog("auth got 'R'\n");
 
-				areq = SOCK_get_int(sock, 4);
-				if (areq == AUTH_REQ_CRYPT)
-					SOCK_get_n_char(sock, salt, 2);
+			switch (beresp)
+			{
+				case 'E':
 
-				mylog("areq = %d\n", areq);
-			}
-			switch(areq) {
-			case AUTH_REQ_OK:
-				break;
+					SOCK_get_string(sock, msgbuffer, ERROR_MSG_LENGTH);
+					CC_set_error(self, CONN_INVALID_AUTHENTICATION, msgbuffer);
+					qlog("ERROR from backend during authentication: '%s'\n", msgbuffer);
+					if (strncmp(msgbuffer, "Unsupported frontend protocol", 29) == 0)
+					{			/* retry older version */
+						if (PROTOCOL_63(ci))
+							strcpy(ci->protocol, PG62);
+						else
+							strcpy(ci->protocol, PG63);
+						SOCK_Destructor(sock);
+						self->sock = (SocketClass *) 0;
+						CC_initialize_pg_version(self);
+						goto another_version_retry;
+					}
 
-			case AUTH_REQ_KRB4:
-				self->errormsg = "Kerberos 4 authentication not supported";
-				self->errornumber = CONN_AUTH_TYPE_UNSUPPORTED;
-				return 0;
+					return 0;
+				case 'R':
 
-			case AUTH_REQ_KRB5:
-				self->errormsg = "Kerberos 5 authentication not supported";
-				self->errornumber = CONN_AUTH_TYPE_UNSUPPORTED;
-				return 0;
+					if (password_req != AUTH_REQ_OK)
+					{
+						mylog("in 'R' password_req=%s\n", ci->password);
+						areq = password_req;
+						if (salt_para)
+							memcpy(salt, salt_para, sizeof(salt));
+						password_req = AUTH_REQ_OK;
+					}
+					else
+					{
 
-			case AUTH_REQ_PASSWORD:
-				mylog("in AUTH_REQ_PASSWORD\n");
+						areq = SOCK_get_int(sock, 4);
+						if (areq == AUTH_REQ_MD5)
+							SOCK_get_n_char(sock, salt, 4);
+						else if (areq == AUTH_REQ_CRYPT)
+							SOCK_get_n_char(sock, salt, 2);
 
-				if (ci->password[0] == '\0') {
-					self->errornumber = CONNECTION_NEED_PASSWORD;
-					self->errormsg = "A password is required for this connection.";
-					return -1;	/* need password */
-				}
+						mylog("areq = %d\n", areq);
+					}
+					switch (areq)
+					{
+						case AUTH_REQ_OK:
+							break;
 
-				mylog("past need password\n");
+						case AUTH_REQ_KRB4:
+							CC_set_error(self, CONN_AUTH_TYPE_UNSUPPORTED, "Kerberos 4 authentication not supported");
+							return 0;
 
-				SOCK_put_int(sock, 4+strlen(ci->password)+1, 4);
-				SOCK_put_n_char(sock, ci->password, strlen(ci->password) + 1);
-				SOCK_flush_output(sock);
+						case AUTH_REQ_KRB5:
+							CC_set_error(self, CONN_AUTH_TYPE_UNSUPPORTED, "Kerberos 5 authentication not supported");
+							return 0;
 
-				mylog("past flush\n");
-				break;
+						case AUTH_REQ_PASSWORD:
+							mylog("in AUTH_REQ_PASSWORD\n");
 
-			case AUTH_REQ_CRYPT:
+							if (ci->password[0] == '\0')
+							{
+								CC_set_error(self, CONNECTION_NEED_PASSWORD, "A password is required for this connection.");
+								return -areq;		/* need password */
+							}
+
+							mylog("past need password\n");
+
+							SOCK_put_int(sock, 4 + strlen(ci->password) + 1, 4);
+							SOCK_put_n_char(sock, ci->password, strlen(ci->password) + 1);
+							SOCK_flush_output(sock);
+
+							mylog("past flush\n");
+							break;
+
+						case AUTH_REQ_CRYPT:
 #ifdef HAVE_LIBCRYPT
-                {
-                    char *password;
+                			{
+                    			char *password;
 
-                    mylog("in AUTH_REQ_CRYPT\n");
+                    			mylog("in AUTH_REQ_CRYPT\n");
 
-                    if (ci->password[0] == '\0') {
-                        self->errornumber = CONNECTION_NEED_PASSWORD;
-                        self->errormsg = "A password is required for this connection.";
-                        return -1;	/* need password */
-                    }
+                    			if (ci->password[0] == '\0') {
+									CC_set_error(self, CONNECTION_NEED_PASSWORD, "A password is required for this connection.");
+                        			return -1;	/* need password */
+                    			}
 
-                    mylog("past need password\n");
-
-                    password = crypt( ci -> password, salt );
-
-                    SOCK_put_int(sock, 4+strlen(password)+1, 4);
-                    SOCK_put_n_char(sock, password, strlen(password) + 1);
-                    SOCK_flush_output(sock);
-
-                    mylog("past flush\n");
-                    break;
-                }
+                    			mylog("past need password\n");
+			
+                    			password = crypt( ci -> password, salt );
+			
+                    			SOCK_put_int(sock, 4+strlen(password)+1, 4);
+                    			SOCK_put_n_char(sock, password, strlen(password) + 1);
+                    			SOCK_flush_output(sock);
+			
+                    			mylog("past flush\n");
+                    			break;
+                			}
 #else
-				self->errormsg = "Password crypt authentication not supported";
-				self->errornumber = CONN_AUTH_TYPE_UNSUPPORTED;
+							CC_set_error(self, CONN_AUTH_TYPE_UNSUPPORTED, "Password crypt authentication not supported.");
 #endif
-				return 0;
+							return 0;
+						case AUTH_REQ_MD5:
+							mylog("in AUTH_REQ_MD5\n");
+							if (ci->password[0] == '\0')
+							{
+								CC_set_error(self, CONNECTION_NEED_PASSWORD, "A password is required for this connection.");
+								if (salt_para)
+									memcpy(salt_para, salt, sizeof(salt));
+								return -areq; /* need password */
+							}
+							if (md5_auth_send(self, salt))
+							{
+								CC_set_error(self, CONN_INVALID_AUTHENTICATION, "md5 hashing failed");
+								return 0;
+							}
+							break;
 
-			default:
-				self->errormsg = "Unknown authentication type";
-				self->errornumber = CONN_AUTH_TYPE_UNSUPPORTED;
-				return 0;
+						case AUTH_REQ_SCM_CREDS:
+							CC_set_error(self, CONN_AUTH_TYPE_UNSUPPORTED, "Unix socket credential authentication not supported");
+							return 0;
+
+						default:
+							CC_set_error(self, CONN_AUTH_TYPE_UNSUPPORTED, "Unknown authentication type");
+							return 0;
+					}
+					break;
+				case 'K':		/* Secret key (6.4 protocol) */
+					self->be_pid = SOCK_get_int(sock, 4);		/* pid */
+					self->be_key = SOCK_get_int(sock, 4);		/* key */
+
+					break;
+				case 'Z':		/* Backend is ready for new query (6.4) */
+					ReadyForQuery = TRUE;
+					break;
+				case 'N':	/* Notices may come */
+					while (SOCK_get_string(sock, notice, sizeof(notice) - 1)) ;
+					break;
+				default:
+					CC_set_error(self, CONN_INVALID_AUTHENTICATION, "Unexpected protocol character during authentication");
+					return 0;
 			}
-			break;
-		default:
-			self->errormsg = "Unexpected protocol character during authentication";
-			self->errornumber = CONN_INVALID_AUTHENTICATION;
-			return 0;
-		}
 
-	} while (areq != AUTH_REQ_OK);		
+			/*
+			 * There were no ReadyForQuery responce before 6.4.
+			 */
+			if (before_64 && areq == AUTH_REQ_OK)
+				ReadyForQuery = TRUE;
+		} while (!ReadyForQuery);
+	}
 
 
 	CC_clear_error(self);	/* clear any password error */
@@ -726,8 +808,7 @@ static char* const func="CC_connect";
 	res = CC_send_query(self, " ", NULL);
 	if ( res == NULL || QR_get_status(res) != PGRES_EMPTY_QUERY) {
 		mylog("got no result from the empty query.  (probably database does not exist)\n");
-		self->errornumber = CONNECTION_NO_SUCH_DATABASE;
-		self->errormsg = "The database does not exist on the server\nor user authentication failed.";
+		CC_set_error(self, CONNECTION_NO_SUCH_DATABASE, "The database does not exist on the server\nor user authentication failed.");
 		if (res != NULL)
 			QR_Destructor(res);
 		return 0;
@@ -768,7 +849,7 @@ int i;
 	mylog("CC_add_statement: self=%u, stmt=%u\n", self, stmt);
 
 	for (i = 0; i < self->num_stmts; i++) {
-		if ( ! self->stmts[i]) {
+		if ( !self->stmts[i]) {
 			stmt->hdbc = self;
 			self->stmts[i] = stmt;
 			return TRUE;
@@ -811,16 +892,16 @@ int i;
 char *
 CC_create_errormsg(ConnectionClass *self)
 {
-SocketClass *sock = self->sock;
-int pos;
-static char msg[4096];
+	SocketClass	*sock = self->sock;
+	int		pos;
+	static char	msg[4096];
 
 	mylog("enter CC_create_errormsg\n");
 
 	msg[0] = '\0';
 
-	if (self->errormsg)
-		strcpy(msg, self->errormsg);
+	if (CC_get_errormsg(self))
+		strncpy(msg, CC_get_errormsg(self), sizeof(msg));
 
 	mylog("msg = '%s'\n", msg);
 
@@ -830,30 +911,52 @@ static char msg[4096];
 	}
 
 	mylog("exit CC_create_errormsg\n");
-	return msg;
+	return msg ? strdup(msg) : NULL;
 }
 
 
+void
+CC_set_error(ConnectionClass *self, int number, const char *message)
+{
+	if (self->__error_message)
+		free(self->__error_message);
+	self->__error_number = number;
+	self->__error_message = message ? strdup(message) : NULL;
+}
+
+
+void
+CC_set_errormsg(ConnectionClass *self, const char *message)
+{
+	if (self->__error_message)
+		free(self->__error_message);
+	self->__error_message = message ? strdup(message) : NULL;
+}
+
 char 
 CC_get_error(ConnectionClass *self, int *number, char **message)
 {
-int rv;
+	int rv;
+	char *msgcrt;
 
 	mylog("enter CC_get_error\n");
 
 	/*	Create a very informative errormsg if it hasn't been done yet. */
 	if ( ! self->errormsg_created) {
-		self->errormsg = CC_create_errormsg(self);
+		msgcrt = CC_create_errormsg(self);
+		if (self->__error_message)
+			free(self->__error_message);
+		self->__error_message = msgcrt;
 		self->errormsg_created = TRUE;
 	}
 
-	if (self->errornumber) {
-		*number = self->errornumber;
-		*message = self->errormsg;
+	if (CC_get_errornumber(self)) {
+		*number = CC_get_errornumber(self);
+		*message = CC_get_errormsg(self);
 	}
-	rv = (self->errornumber != 0);
+	rv = (CC_get_errornumber(self) != 0);
 
-	self->errornumber = 0;		/* clear the error */
+	self->__error_number = 0;		/* clear the error */
 
 	mylog("exit CC_get_error\n");
 
@@ -885,8 +988,7 @@ char cmdbuffer[MAX_MESSAGE_LEN+1];	/* QR_set_command() dups this string so dont
 
 	/* Indicate that we are sending a query to the backend */
 	if(strlen(query) > MAX_MESSAGE_LEN-2) {
-		self->errornumber = CONNECTION_MSG_TOO_LONG;
-		self->errormsg = "Query string is too long";
+		CC_set_error(self, CONNECTION_MSG_TOO_LONG, "Query string is too long");
 		return NULL;
 	}
 
@@ -894,16 +996,14 @@ char cmdbuffer[MAX_MESSAGE_LEN+1];	/* QR_set_command() dups this string so dont
 		return NULL;
 
 	if (SOCK_get_errcode(sock) != 0) {
-		self->errornumber = CONNECTION_COULD_NOT_SEND;
-		self->errormsg = "Could not send Query to backend";
+		CC_set_error(self, CONNECTION_COULD_NOT_SEND, "Could not send Query to backend");
 		CC_set_no_trans(self);
 		return NULL;
 	}
 
 	SOCK_put_char(sock, 'Q');
 	if (SOCK_get_errcode(sock) != 0) {
-		self->errornumber = CONNECTION_COULD_NOT_SEND;
-		self->errormsg = "Could not send Query to backend";
+		CC_set_error(self, CONNECTION_COULD_NOT_SEND, "Could not send Query to backend");
 		CC_set_no_trans(self);
 		return NULL;
 	}
@@ -912,8 +1012,7 @@ char cmdbuffer[MAX_MESSAGE_LEN+1];	/* QR_set_command() dups this string so dont
 	SOCK_flush_output(sock);
 
 	if (SOCK_get_errcode(sock) != 0) {
-		self->errornumber = CONNECTION_COULD_NOT_SEND;
-		self->errormsg = "Could not send Query to backend";
+		CC_set_error(self, CONNECTION_COULD_NOT_SEND, "Could not send Query to backend");
 		CC_set_no_trans(self);
 		return NULL;
 	}
@@ -925,12 +1024,11 @@ char cmdbuffer[MAX_MESSAGE_LEN+1];	/* QR_set_command() dups this string so dont
 		id = SOCK_get_char(sock);
 
 		if ((SOCK_get_errcode(sock) != 0) || (id == EOF)) {
-			self->errornumber = CONNECTION_NO_RESPONSE;
-			self->errormsg = "No response from the backend";
+			CC_set_error(self, CONNECTION_NO_RESPONSE, "No response from the backend");
 			if (res)
 				QR_Destructor(res);
 
-			mylog("send_query: 'id' - %s\n", self->errormsg);
+			mylog("send_query: 'id' - %s\n", CC_get_errormsg(self));
 			CC_set_no_trans(self);
 			return NULL;
 		}
@@ -947,9 +1045,8 @@ char cmdbuffer[MAX_MESSAGE_LEN+1];	/* QR_set_command() dups this string so dont
 			/* read in the return message from the backend */
 			SOCK_get_string(sock, cmdbuffer, MAX_MESSAGE_LEN);
 			if (SOCK_get_errcode(sock) != 0) {
-				self->errornumber = CONNECTION_NO_RESPONSE;
-				self->errormsg = "No response from backend while receiving a portal query command";
-				mylog("send_query: 'C' - %s\n", self->errormsg);
+				CC_set_error(self, CONNECTION_NO_RESPONSE, "No response from backend while receiving a portal query command");
+				mylog("send_query: 'C' - %s\n", CC_get_errormsg(self));
 				CC_set_no_trans(self);
                 if (res)
                     QR_Destructor(res);
@@ -961,7 +1058,10 @@ char cmdbuffer[MAX_MESSAGE_LEN+1];	/* QR_set_command() dups this string so dont
 				mylog("send_query: ok - 'C' - %s\n", cmdbuffer);
 
                 if (res)
+                {
                     QR_Destructor(res);
+                    res = NULL;
+                }
                 
 				if (res == NULL)	/* allow for "show" style notices */
 					res = QR_Constructor();
@@ -1008,13 +1108,13 @@ char cmdbuffer[MAX_MESSAGE_LEN+1];	/* QR_set_command() dups this string so dont
 						   error reporting, from PostgreSQL 7.0).
 						   (Zoltan Kovacs, 04/26/2000)
 						*/
-						self->errormsg = cmdbuffer;
-						if ( ! strncmp(self->errormsg, "FATAL", 5)) {
-						    self->errornumber = CONNECTION_SERVER_REPORTED_ERROR;
+						CC_set_errormsg(self, cmdbuffer);
+						if ( ! strncmp(cmdbuffer, "FATAL", 5)) {
+						    CC_set_errornumber(self, CONNECTION_SERVER_REPORTED_ERROR);
 						    CC_set_no_trans(self);
 						    }
 						else
-						    self->errornumber = CONNECTION_SERVER_REPORTED_WARNING;
+						    CC_set_errornumber(self, CONNECTION_SERVER_REPORTED_WARNING);
 						QR_set_status(res, PGRES_NONFATAL_ERROR);
 						QR_set_aborted(res, TRUE);
 						break;
@@ -1049,8 +1149,7 @@ char cmdbuffer[MAX_MESSAGE_LEN+1];	/* QR_set_command() dups this string so dont
 				/* There is a closing '\0' following the 'I', so we eat it */
 			swallow = SOCK_get_char(sock);
 			if ((swallow != '\0') || SOCK_get_errcode(sock) != 0) {
-				self->errornumber = CONNECTION_BACKEND_CRAZY;
-				self->errormsg = "Unexpected protocol character from backend (send_query - I)";
+				CC_set_error(self, CONNECTION_BACKEND_CRAZY, "Unexpected protocol character from backend (send_query - I)");
                 if (res)
                     QR_Destructor(res);
 				res = QR_Constructor();
@@ -1072,23 +1171,23 @@ char cmdbuffer[MAX_MESSAGE_LEN+1];	/* QR_set_command() dups this string so dont
 			if (msgbuffer[0] != '\0' && msgbuffer[strlen(msgbuffer)-1] == '\n')
 				msgbuffer[strlen(msgbuffer)-1] = '\0';
 
-			self->errormsg = msgbuffer;
+			CC_set_errormsg(self, msgbuffer);
 
-			mylog("send_query: 'E' - %s\n", self->errormsg);
-			qlog("ERROR from backend during send_query: '%s'\n", self->errormsg);
+			mylog("send_query: 'E' - %s\n", msgbuffer);
+			qlog("ERROR from backend during send_query: '%s'\n", msgbuffer);
 
 			/* We should report that an error occured. Zoltan */
             if (res)
                 QR_Destructor(res);
 			res = QR_Constructor();
 
-			if ( ! strncmp(self->errormsg, "FATAL", 5)) {
-				self->errornumber = CONNECTION_SERVER_REPORTED_ERROR;
+			if ( ! strncmp(msgbuffer, "FATAL", 5)) {
+				CC_set_errornumber(self, CONNECTION_SERVER_REPORTED_ERROR);
 				CC_set_no_trans(self);
 				QR_set_status(res, PGRES_FATAL_ERROR);
 			}
 			else {
-				self->errornumber = CONNECTION_SERVER_REPORTED_WARNING;
+				CC_set_errornumber(self, CONNECTION_SERVER_REPORTED_WARNING);
 				QR_set_status(res, PGRES_NONFATAL_ERROR);
 			}
 			QR_set_aborted(res, TRUE);
@@ -1105,8 +1204,7 @@ char cmdbuffer[MAX_MESSAGE_LEN+1];	/* QR_set_command() dups this string so dont
 				result_in = QR_Constructor();
 				mylog("send_query: 'T' no result_in: res = %u\n", result_in);
 				if ( ! result_in) {
-					self->errornumber = CONNECTION_COULD_NOT_RECEIVE;
-					self->errormsg = "Could not create result info in send_query.";
+					CC_set_error(self, CONNECTION_COULD_NOT_RECEIVE, "Could not create result info in send_query.");
                     if (res)
                         QR_Destructor(res);
 					return NULL;
@@ -1116,17 +1214,16 @@ char cmdbuffer[MAX_MESSAGE_LEN+1];	/* QR_set_command() dups this string so dont
 					QR_set_cache_size(result_in, qi->row_size);
 
 				if ( ! QR_fetch_tuples(result_in, self, qi ? qi->cursor : NULL)) {
-					self->errornumber = CONNECTION_COULD_NOT_RECEIVE;
+					CC_set_errornumber(self, CONNECTION_COULD_NOT_RECEIVE);
                     if (res)
                         QR_Destructor(res);
-					self->errormsg = QR_get_message(result_in);
+					CC_set_errormsg(self, QR_get_message(result_in));
 					return NULL;
 				}
 			}
 			else {  /* next fetch, so reuse an existing result */
 				if ( ! QR_fetch_tuples(result_in, NULL, NULL)) {
-					self->errornumber = CONNECTION_COULD_NOT_RECEIVE;
-					self->errormsg = QR_get_message(result_in);
+					CC_set_error(self, CONNECTION_COULD_NOT_RECEIVE, QR_get_message(result_in));
                     if (res)
                         QR_Destructor(res);
 					return NULL;
@@ -1135,25 +1232,24 @@ char cmdbuffer[MAX_MESSAGE_LEN+1];	/* QR_set_command() dups this string so dont
 
 			return result_in;
 		case 'D': /* Copy in command began successfully */
-            if (res)
-                QR_Destructor(res);
+			if (res)
+			QR_Destructor(res);
 			res = QR_Constructor();
 			QR_set_status(res, PGRES_COPY_IN);
 			return res;
 		case 'B': /* Copy out command began successfully */
-            if (res)
-                QR_Destructor(res);
+			if (res)
+			QR_Destructor(res);
 			res = QR_Constructor();
 			QR_set_status(res, PGRES_COPY_OUT);
 			return res;
 		default:
-			self->errornumber = CONNECTION_BACKEND_CRAZY;
-			self->errormsg = "Unexpected protocol character from backend (send_query)";
+			CC_set_error(self, CONNECTION_BACKEND_CRAZY, "Unexpected protocol character from backend (send_query)");
 			CC_set_no_trans(self);
 
-			mylog("send_query: error - %s\n", self->errormsg);
-            if (res)
-                QR_Destructor(res);
+			mylog("send_query: error - %s\n", CC_get_errormsg(self));
+			if (res)
+			QR_Destructor(res);
 			return NULL;
 		}
 	}
@@ -1170,16 +1266,14 @@ int i;
 	mylog("send_function(): conn=%u, fnid=%d, result_is_int=%d, nargs=%d\n", self, fnid, result_is_int, nargs);
 
 	if (SOCK_get_errcode(sock) != 0) {
-		self->errornumber = CONNECTION_COULD_NOT_SEND;
-		self->errormsg = "Could not send function to backend";
+		CC_set_error(self, CONNECTION_COULD_NOT_SEND, "Could not send function to backend");
 		CC_set_no_trans(self);
 		return FALSE;
 	}
 
 	SOCK_put_string(sock, "F ");
 	if (SOCK_get_errcode(sock) != 0) {
-		self->errornumber = CONNECTION_COULD_NOT_SEND;
-		self->errormsg = "Could not send function to backend";
+		CC_set_error(self, CONNECTION_COULD_NOT_SEND, "Could not send function to backend");
 		CC_set_no_trans(self);
 		return FALSE;
 	}
@@ -1226,10 +1320,10 @@ int i;
 
 		case 'E':
 			SOCK_get_string(sock, msgbuffer, ERROR_MSG_LENGTH);
-			self->errormsg = msgbuffer;
+			CC_set_errormsg(self, msgbuffer);
 
-			mylog("send_function(V): 'E' - %s\n", self->errormsg);
-			qlog("ERROR from backend during send_function: '%s'\n", self->errormsg);
+			mylog("send_function(V): 'E' - %s\n", msgbuffer);
+			qlog("ERROR from backend during send_function: '%s'\n", msgbuffer);
 
 			return FALSE;
 
@@ -1237,11 +1331,10 @@ int i;
 			break;
 
 		default:
-			self->errornumber = CONNECTION_BACKEND_CRAZY;
-			self->errormsg = "Unexpected protocol character from backend (send_function, args)";
+			CC_set_error(self, CONNECTION_BACKEND_CRAZY, "Unexpected protocol character from backend (send_function, args)");
 			CC_set_no_trans(self);
 
-			mylog("send_function: error - %s\n", self->errormsg);
+			mylog("send_function: error - %s\n", CC_get_errormsg(self));
 			return FALSE;
 		}
 	}
@@ -1270,10 +1363,10 @@ int i;
 
 		case 'E':
 			SOCK_get_string(sock, msgbuffer, ERROR_MSG_LENGTH);
-			self->errormsg = msgbuffer;
+			CC_set_errormsg(self, msgbuffer);
 
-			mylog("send_function(G): 'E' - %s\n", self->errormsg);
-			qlog("ERROR from backend during send_function: '%s'\n", self->errormsg);
+			mylog("send_function(G): 'E' - %s\n", msgbuffer);
+			qlog("ERROR from backend during send_function: '%s'\n", msgbuffer);
 
 			return FALSE;
 
@@ -1289,11 +1382,10 @@ int i;
 			return TRUE;
 
 		default:
-			self->errornumber = CONNECTION_BACKEND_CRAZY;
-			self->errormsg = "Unexpected protocol character from backend (send_function, result)";
+			CC_set_error(self, CONNECTION_BACKEND_CRAZY, "Unexpected protocol character from backend (send_function, result)");
 			CC_set_no_trans(self);
 
-			mylog("send_function: error - %s\n", self->errormsg);
+			mylog("send_function: error - %s\n", CC_get_errormsg(self));
 			return FALSE;
 		}
 	}
@@ -1534,8 +1626,8 @@ CC_log_error(char *func, char *desc, ConnectionClass *self)
 #endif
 
 	if (self) {
-		qlog("CONN ERROR: func=%s, desc='%s', errnum=%d, errmsg='%s'\n", func, desc, self->errornumber, nullcheck (self->errormsg));
-		mylog("CONN ERROR: func=%s, desc='%s', errnum=%d, errmsg='%s'\n", func, desc, self->errornumber, nullcheck (self->errormsg));
+		qlog("CONN ERROR: func=%s, desc='%s', errnum=%d, errmsg='%s'\n", func, desc, self->__error_number, nullcheck (self->__error_message));
+		mylog("CONN ERROR: func=%s, desc='%s', errnum=%d, errmsg='%s'\n", func, desc, self->__error_number, nullcheck (self->__error_message));
 		qlog("            ------------------------------------------------------------\n");
 		qlog("            henv=%u, conn=%u, status=%u, num_stmts=%d\n", self->henv, self, self->status, self->num_stmts);
 		qlog("            sock=%u, stmts=%u, lobj_type=%d\n", self->sock, self->stmts, self->lobj_type);
