@@ -655,6 +655,30 @@ struct tm tp;
 			}
 			break;
 
+#ifdef HAVE_LONG_LONG
+		case SQL_BIGINT:
+            {
+            long long lv;
+
+			len = 8;
+
+#ifdef HAVE_ATOLL
+            lv = atoll( value );
+#elif HAVE_STROLL
+            lv = strtoll( value, NULL, 10 );
+#else
+            lv = atol( value );
+#endif
+
+			if (bind_size > 0) {
+				*(long long *) ((char *) rgbValue + (bind_row * bind_size)) = lv; 
+			} else {
+				*((long long *)rgbValue + bind_row) = lv;
+			}
+            }
+			break;
+#endif
+
 		case SQL_C_BINARY:	
 
 			/*	truncate if necessary */
@@ -936,7 +960,11 @@ int lobj_fd, retval;
 
 		case SQL_C_SLONG:
 		case SQL_C_LONG:
+#if (SIZEOF_LONG == 4)
 			sprintf(param_string, "%ld",
+#else
+			sprintf(param_string, "%d",
+#endif
 				*((SDWORD *) buffer));
 			break;
 
@@ -953,7 +981,11 @@ int lobj_fd, retval;
 			break;
 
 		case SQL_C_ULONG:
+#if (SIZEOF_LONG == 4)
 			sprintf(param_string, "%lu",
+#else
+			sprintf(param_string, "%u",
+#endif
 				*((UDWORD *) buffer));
 			break;
 
@@ -1011,8 +1043,7 @@ int lobj_fd, retval;
 							  }
 		default:
 			/* error */
-			stmt->errormsg = "Unrecognized C_parameter type in copy_statement_with_parameters";
-			stmt->errornumber = STMT_NOT_IMPLEMENTED_ERROR;
+			SC_set_error(stmt, STMT_NOT_IMPLEMENTED_ERROR, "Unrecognized C_parameter type in copy_statement_with_parameters");
 			new_statement[npos] = '\0';   /* just in case */
 			SC_log_error(func, "", stmt);
 			free(cbuf);
@@ -1124,8 +1155,7 @@ int lobj_fd, retval;
 
 					res = CC_send_query(stmt->hdbc, "BEGIN", NULL);
 					if (!res) {
-						stmt->errormsg = "Could not begin (in-line) a transaction";
-						stmt->errornumber = STMT_EXEC_ERROR;
+						SC_set_error(stmt, STMT_EXEC_ERROR, "Could not begin (in-line) a transaction");
 						SC_log_error(func, "", stmt);
 						free(cbuf);
 						return SQL_ERROR;
@@ -1133,8 +1163,7 @@ int lobj_fd, retval;
 					ok = QR_command_successful(res);
 					QR_Destructor(res);
 					if (!ok) {
-						stmt->errormsg = "Could not begin (in-line) a transaction";
-						stmt->errornumber = STMT_EXEC_ERROR;
+						SC_set_error(stmt, STMT_EXEC_ERROR, "Could not begin (in-line) a transaction");
 						SC_log_error(func, "", stmt);
 						free(cbuf);
 						return SQL_ERROR;
@@ -1144,28 +1173,26 @@ int lobj_fd, retval;
 				}
 
 				/*	store the oid */
-				lobj_oid = lo_creat(stmt->hdbc, INV_READ | INV_WRITE);
+				lobj_oid = odbc_lo_creat(stmt->hdbc, INV_READ | INV_WRITE);
 				if (lobj_oid == 0) {
-					stmt->errornumber = STMT_EXEC_ERROR;
-					stmt->errormsg = "Couldnt create (in-line) large object.";
+					SC_set_error(stmt, STMT_EXEC_ERROR, "Couldnt create (in-line) large object.");
 					SC_log_error(func, "", stmt);
 					free(cbuf);
 					return SQL_ERROR;
 				}
 
 				/*	store the fd */
-				lobj_fd = lo_open(stmt->hdbc, lobj_oid, INV_WRITE);
+				lobj_fd = odbc_lo_open(stmt->hdbc, lobj_oid, INV_WRITE);
 				if ( lobj_fd < 0) {
-					stmt->errornumber = STMT_EXEC_ERROR;
-					stmt->errormsg = "Couldnt open (in-line) large object for writing.";
+					SC_set_error(stmt, STMT_EXEC_ERROR, "Couldnt open (in-line) large object for writing.");
 					SC_log_error(func, "", stmt);
 					free(cbuf);
 					return SQL_ERROR;
 				}
 
-				retval = lo_write(stmt->hdbc, lobj_fd, buffer, used);
+				retval = odbc_lo_write(stmt->hdbc, lobj_fd, buffer, used);
 
-				lo_close(stmt->hdbc, lobj_fd);
+				odbc_lo_close(stmt->hdbc, lobj_fd);
 
 				/* commit transaction if needed */
 				if (!globals.use_declarefetch && CC_is_in_autocommit(stmt->hdbc)) {
@@ -1174,8 +1201,7 @@ int lobj_fd, retval;
 
 					res = CC_send_query(stmt->hdbc, "COMMIT", NULL);
 					if (!res) {
-						stmt->errormsg = "Could not commit (in-line) a transaction";
-						stmt->errornumber = STMT_EXEC_ERROR;
+						SC_set_error(stmt, STMT_EXEC_ERROR, "Could not commit (in-line) a transaction");
 						SC_log_error(func, "", stmt);
 						free(cbuf);
 						return SQL_ERROR;
@@ -1183,8 +1209,7 @@ int lobj_fd, retval;
 					ok = QR_command_successful(res);
 					QR_Destructor(res);
 					if (!ok) {
-						stmt->errormsg = "Could not commit (in-line) a transaction";
-						stmt->errornumber = STMT_EXEC_ERROR;
+						SC_set_error(stmt, STMT_EXEC_ERROR, "Could not commit (in-line) a transaction");
 						SC_log_error(func, "", stmt);
 						free(cbuf);
 						return SQL_ERROR;
@@ -1694,15 +1719,13 @@ convert_lo(StatementClass *stmt, void *value, Int2 fCType, PTR rgbValue,
 
 			res = CC_send_query(stmt->hdbc, "BEGIN", NULL);
 			if (!res) {
-				stmt->errormsg = "Could not begin (in-line) a transaction";
-				stmt->errornumber = STMT_EXEC_ERROR;
+				SC_set_error(stmt, STMT_EXEC_ERROR, "Could not begin (in-line) a transaction");
 				return COPY_GENERAL_ERROR;
 			}
 			ok = QR_command_successful(res);
 			QR_Destructor(res);
 			if (!ok) {
-				stmt->errormsg = "Could not begin (in-line) a transaction";
-				stmt->errornumber = STMT_EXEC_ERROR;
+				SC_set_error(stmt, STMT_EXEC_ERROR, "Could not begin (in-line) a transaction");
 				return COPY_GENERAL_ERROR;
 			}
 
@@ -1710,23 +1733,22 @@ convert_lo(StatementClass *stmt, void *value, Int2 fCType, PTR rgbValue,
 		}
 
 		oid = atoi(value);
-		stmt->lobj_fd = lo_open(stmt->hdbc, oid, INV_READ);
+		stmt->lobj_fd = odbc_lo_open(stmt->hdbc, oid, INV_READ);
 		if (stmt->lobj_fd < 0) {
-			stmt->errornumber = STMT_EXEC_ERROR;
-			stmt->errormsg = "Couldnt open large object for reading.";
+			SC_set_error(stmt, STMT_EXEC_ERROR, "Couldnt open large object for reading.");
 			return COPY_GENERAL_ERROR;
 		}
 
 		/*	Get the size */
-		retval = lo_lseek(stmt->hdbc, stmt->lobj_fd, 0L, SEEK_END);
+		retval = odbc_lo_lseek(stmt->hdbc, stmt->lobj_fd, 0L, SEEK_END);
 		if (retval >= 0) {
 
-			left = lo_tell(stmt->hdbc, stmt->lobj_fd);
+			left = odbc_lo_tell(stmt->hdbc, stmt->lobj_fd);
 			if (bindInfo)
 				bindInfo->data_left = left;
 
 			/*	return to beginning */
-			lo_lseek(stmt->hdbc, stmt->lobj_fd, 0L, SEEK_SET);
+			odbc_lo_lseek(stmt->hdbc, stmt->lobj_fd, 0L, SEEK_SET);
 		}
 	}
 
@@ -1735,14 +1757,13 @@ convert_lo(StatementClass *stmt, void *value, Int2 fCType, PTR rgbValue,
 	}
 
 	if (stmt->lobj_fd < 0) {
-		stmt->errornumber = STMT_EXEC_ERROR;
-		stmt->errormsg = "Large object FD undefined for multiple read.";
+		SC_set_error(stmt, STMT_EXEC_ERROR, "Large object FD undefined for multiple read.");
 		return COPY_GENERAL_ERROR;
 	}
 
-	retval = lo_read(stmt->hdbc, stmt->lobj_fd, (char *) rgbValue, cbValueMax);
+	retval = odbc_lo_read(stmt->hdbc, stmt->lobj_fd, (char *) rgbValue, cbValueMax);
 	if (retval < 0) {
-		lo_close(stmt->hdbc, stmt->lobj_fd);
+		odbc_lo_close(stmt->hdbc, stmt->lobj_fd);
 
 		/* commit transaction if needed */
 		if (!globals.use_declarefetch && CC_is_in_autocommit(stmt->hdbc)) {
@@ -1751,15 +1772,13 @@ convert_lo(StatementClass *stmt, void *value, Int2 fCType, PTR rgbValue,
 
 			res = CC_send_query(stmt->hdbc, "COMMIT", NULL);
 			if (!res) {
-				stmt->errormsg = "Could not commit (in-line) a transaction";
-				stmt->errornumber = STMT_EXEC_ERROR;
+				SC_set_error(stmt, STMT_EXEC_ERROR, "Could not commit (in-line) a transaction");
 				return COPY_GENERAL_ERROR;
 			}
 			ok = QR_command_successful(res);
 			QR_Destructor(res);
 			if (!ok) {
-				stmt->errormsg = "Could not commit (in-line) a transaction";
-				stmt->errornumber = STMT_EXEC_ERROR;
+				SC_set_error(stmt, STMT_EXEC_ERROR, "Could not commit (in-line) a transaction");
 				return COPY_GENERAL_ERROR;
 			}
 
@@ -1768,8 +1787,7 @@ convert_lo(StatementClass *stmt, void *value, Int2 fCType, PTR rgbValue,
 
 		stmt->lobj_fd = -1;
 
-		stmt->errornumber = STMT_EXEC_ERROR;
-		stmt->errormsg = "Error reading from large object.";
+		SC_set_error(stmt, STMT_EXEC_ERROR, "Error reading from large object.");
 		return COPY_GENERAL_ERROR;
 	}
 
@@ -1787,7 +1805,7 @@ convert_lo(StatementClass *stmt, void *value, Int2 fCType, PTR rgbValue,
 
 
 	if (! bindInfo || bindInfo->data_left == 0) {
-		lo_close(stmt->hdbc, stmt->lobj_fd);
+		odbc_lo_close(stmt->hdbc, stmt->lobj_fd);
 
 		/* commit transaction if needed */
 		if (!globals.use_declarefetch && CC_is_in_autocommit(stmt->hdbc)) {
@@ -1796,15 +1814,13 @@ convert_lo(StatementClass *stmt, void *value, Int2 fCType, PTR rgbValue,
 
 			res = CC_send_query(stmt->hdbc, "COMMIT", NULL);
 			if (!res) {
-				stmt->errormsg = "Could not commit (in-line) a transaction";
-				stmt->errornumber = STMT_EXEC_ERROR;
+				SC_set_error(stmt, STMT_EXEC_ERROR, "Could not commit (in-line) a transaction");
 				return COPY_GENERAL_ERROR;
 			}
 			ok = QR_command_successful(res);
 			QR_Destructor(res);
 			if (!ok) {
-				stmt->errormsg = "Could not commit (in-line) a transaction";
-				stmt->errornumber = STMT_EXEC_ERROR;
+				SC_set_error(stmt, STMT_EXEC_ERROR, "Could not commit (in-line) a transaction");
 				return COPY_GENERAL_ERROR;
 			}
 
