@@ -81,19 +81,17 @@ StatementClass *stmt;
 	mylog("**** SQLAllocStmt: hdbc = %u, stmt = %u\n", hdbc, stmt);
 
 	if ( ! stmt) {
-		conn->errornumber = CONN_STMT_ALLOC_ERROR;
-		conn->errormsg = "No more memory to allocate a further SQL-statement";
+		CC_set_error(conn, CONN_STMT_ALLOC_ERROR, "No more memory to allocate a further SQL-statement");
 		*phstmt = SQL_NULL_HSTMT;
 		CC_log_error(func, "", conn);
 		return SQL_ERROR;
 	}
 
     if ( ! CC_add_statement(conn, stmt)) {
-	conn->errormsg = "Maximum number of connections exceeded.";
-	conn->errornumber = CONN_STMT_ALLOC_ERROR;
-		CC_log_error(func, "", conn);
+	CC_set_error(conn, CONN_STMT_ALLOC_ERROR, "Maximum number of connections exceeded.");
+	CC_log_error(func, "", conn);
 	SC_Destructor(stmt);
-		*phstmt = SQL_NULL_HSTMT;
+	*phstmt = SQL_NULL_HSTMT;
 	return SQL_ERROR;
     }
 
@@ -103,7 +101,6 @@ StatementClass *stmt;
 	*/
 	stmt->options = conn->stmtOptions;
 
-
 	/*	Save the handle for later */
 	stmt->phstmt = phstmt;
 
@@ -136,8 +133,7 @@ StatementClass *stmt = (StatementClass *) hstmt;
 		/* Remove the statement from the connection's statement list */
 		if ( conn) {
 			if ( ! CC_remove_statement(conn, stmt)) {
-				stmt->errornumber = STMT_SEQUENCE_ERROR;
-				stmt->errormsg = "Statement is currently executing a transaction.";
+				SC_set_error(stmt, STMT_SEQUENCE_ERROR, "Statement is currently executing a transaction.");
 				SC_log_error(func, "", stmt);
 				return SQL_ERROR;  /* stmt may be executing a transaction */
 			}
@@ -168,9 +164,8 @@ StatementClass *stmt = (StatementClass *) hstmt;
 		SC_free_params(stmt, STMT_FREE_PARAMS_ALL);
 
     } else {
-	stmt->errormsg = "Invalid option passed to SQLFreeStmt.";
-	stmt->errornumber = STMT_OPTION_OUT_OF_RANGE_ERROR;
-		SC_log_error(func, "", stmt);
+	SC_set_error(stmt, STMT_OPTION_OUT_OF_RANGE_ERROR, "Invalid option passed to SQLFreeStmt.");
+	SC_log_error(func, "", stmt);
 	return SQL_ERROR;
     }
 
@@ -203,7 +198,7 @@ InitializeStatementOptions(StatementOptions *opt)
 StatementClass *
 SC_Constructor(void)
 {
-StatementClass *rv;
+	StatementClass *rv;
 
 	rv = (StatementClass *) malloc(sizeof(StatementClass));
 	if (rv) {
@@ -215,9 +210,7 @@ StatementClass *rv;
 		rv->status = STMT_ALLOCATED;
 		rv->internal = FALSE;
 
-		rv->errormsg = NULL;
-		rv->errornumber = 0;
-		rv->errormsg_created = FALSE;
+		SC_clear_error(rv);
 
 		rv->statement = NULL;
 		rv->stmt_with_params[0] = '\0';
@@ -266,8 +259,7 @@ SC_Destructor(StatementClass *self)
 
 	mylog("SC_Destructor: self=%u, self->result=%u, self->hdbc=%u\n", self, self->result, self->hdbc);
 	if (STMT_EXECUTING == self->status) {
-		self->errornumber = STMT_SEQUENCE_ERROR;
-		self->errormsg = "Statement is currently executing a transaction.";
+		SC_set_error(self, STMT_SEQUENCE_ERROR, "Statement is currently executing a transaction.");
 		return FALSE;
 	}
 
@@ -389,14 +381,11 @@ mylog("recycle statement: self= %u\n", self);
 
 	/*	This would not happen */
 	if (self->status == STMT_EXECUTING) {
-		self->errornumber = STMT_SEQUENCE_ERROR;
-		self->errormsg = "Statement is currently executing a transaction.";
+		SC_set_error(self, STMT_SEQUENCE_ERROR, "Statement is currently executing a transaction.");
 		return FALSE;
 	}
 
-	self->errormsg = NULL;
-	self->errornumber = 0;
-	self->errormsg_created = FALSE;
+	SC_clear_error(self);
 
 	switch (self->status) {
 	case STMT_ALLOCATED:
@@ -422,8 +411,7 @@ mylog("recycle statement: self= %u\n", self);
 		break;
 
 	default:
-		self->errormsg = "An internal error occured while recycling statements";
-		self->errornumber = STMT_INTERNAL_ERROR;
+		SC_set_error(self, STMT_INTERNAL_ERROR, "An internal error occured while recycling statements");
 		return FALSE;
 	}
 
@@ -470,9 +458,7 @@ mylog("recycle statement: self= %u\n", self);
 	self->bind_row = 0;
 	self->last_fetch_count = 0;
 
-	self->errormsg = NULL;
-	self->errornumber = 0;
-	self->errormsg_created = FALSE;
+	SC_clear_error(self);
 
 	self->lobj_fd = -1;
 
@@ -526,8 +512,8 @@ Int2 lf;
 void
 SC_clear_error(StatementClass *self)
 {
-	self->errornumber = 0;
-	self->errormsg = NULL;
+	self->__error_number = 0;
+	self->__error_message = NULL;
 	self->errormsg_created = FALSE;
 }
 
@@ -537,25 +523,39 @@ SC_clear_error(StatementClass *self)
 char *
 SC_create_errormsg(StatementClass *self)
 {
-QResultClass *res = self->result;
-ConnectionClass *conn = self->hdbc;
-int pos;
-static char msg[4096];
+	QResultClass	*res = self->result;
+	ConnectionClass *conn = self->hdbc;
+	int		pos;
+	BOOL		detailmsg = FALSE;
+	char		msg[4096];
 
 	msg[0] = '\0';
 
-	if (res && res->message)
-		strcpy(msg, res->message);
-
-	else if (self->errormsg)
-		strcpy(msg, self->errormsg);
+	if (res && res->message) {
+		strncpy(msg, res->message, sizeof(msg));
+		detailmsg = TRUE;
+	}
+	else if (SC_get_errormsg(self))
+		strncpy(msg, SC_get_errormsg(self), sizeof(msg));
 
+	if (!msg[0] && res && QR_get_notice(res))
+	{
+		char *notice = QR_get_notice(res);
+		int len = strlen(notice);
+		if (len < sizeof(msg))
+		{
+			memcpy(msg, notice, len);
+			msg[len] = '\0';
+		}
+		else
+			return strdup(notice);
+	}
 	if (conn) {
 		SocketClass *sock = conn->sock;
 
-		if (conn->errormsg && conn->errormsg[0] != '\0') {
+		if (!detailmsg && CC_get_errormsg(conn) && (CC_get_errormsg(conn))[0] != '\0') {
 			pos = strlen(msg);
-			sprintf(&msg[pos], ";\n%s", conn->errormsg);
+			sprintf(&msg[pos], ";\n%s", CC_get_errormsg(conn));
 		}
 
 		if (sock && sock->errormsg && sock->errormsg[0] != '\0') {
@@ -564,9 +564,28 @@ static char msg[4096];
 		}
 	}
 
-	return msg;
+	return msg[0] ? strdup(msg) : NULL;
 }
 
+void
+SC_set_error(StatementClass *self, int number, const char *message)
+{
+	if (self->__error_message)
+		free(self->__error_message);
+	self->__error_number = number;
+	self->__error_message = message ? strdup(message) : NULL;
+}
+
+
+void
+SC_set_errormsg(StatementClass *self, const char *message)
+{
+	if (self->__error_message)
+		free(self->__error_message);
+	self->__error_message = message ? strdup(message) : NULL;
+}
+
+
 char
 SC_get_error(StatementClass *self, int *number, char **message)
 {
@@ -574,18 +593,18 @@ char rv;
 
 /*	Create a very informative errormsg if it hasn't been done yet. */
 	if ( ! self->errormsg_created) {
-		self->errormsg = SC_create_errormsg(self);
+		self->__error_message = SC_create_errormsg(self);
 		self->errormsg_created = TRUE;
 	}
 
-	if ( self->errornumber) {
-		*number = self->errornumber;
-		*message = self->errormsg;
-		self->errormsg = NULL;
+	if (self->__error_number) {
+		*number = self->__error_number;
+		*message = self->__error_message;
+		self->__error_message = NULL;
 	}
 
-	rv = (self->errornumber != 0);
-	self->errornumber = 0;
+	rv = (self->__error_number != 0);
+	self->__error_number = 0;
 
 	return rv;
 }
@@ -645,8 +664,7 @@ ColumnInfoClass *ci;
 
 		else {
 			mylog("SQLFetch: error\n");
-			self->errornumber = STMT_EXEC_ERROR;
-			self->errormsg = "Error fetching next row";
+			SC_set_error(self, STMT_EXEC_ERROR, "Error fetching next row");
 			SC_log_error(func, "", self);
 			return SQL_ERROR;
 		}
@@ -706,22 +724,19 @@ ColumnInfoClass *ci;
 				break;	/*	OK, do next bound column */
 
 			case COPY_UNSUPPORTED_TYPE:
-				self->errormsg = "Received an unsupported type from Postgres.";
-				self->errornumber = STMT_RESTRICTED_DATA_TYPE_ERROR;
+				SC_set_error(self, STMT_RESTRICTED_DATA_TYPE_ERROR, "Received an unsupported type from Postgres.");
 				SC_log_error(func, "", self);
 				result = SQL_ERROR;
 				break;
 
 			case COPY_UNSUPPORTED_CONVERSION:
-				self->errormsg = "Couldn't handle the necessary data type conversion.";
-				self->errornumber = STMT_RESTRICTED_DATA_TYPE_ERROR;
+				SC_set_error(self, STMT_RESTRICTED_DATA_TYPE_ERROR, "Couldn't handle the necessary data type conversion.");
 				SC_log_error(func, "", self);
 				result = SQL_ERROR;
 				break;
 
 			case COPY_RESULT_TRUNCATED:
-				self->errornumber = STMT_TRUNCATED;
-				self->errormsg = "The buffer was too small for the result.";
+				SC_set_error(self, STMT_TRUNCATED, "The buffer was too small for the result.");
 				result = SQL_SUCCESS_WITH_INFO;
 				break;
 
@@ -735,8 +750,7 @@ ColumnInfoClass *ci;
 				break;
 
 			default:
-				self->errormsg = "Unrecognized return value from copy_and_convert_field.";
-				self->errornumber = STMT_INTERNAL_ERROR;
+				SC_set_error(self, STMT_INTERNAL_ERROR, "Unrecognized return value from copy_and_convert_field.");
 				SC_log_error(func, "", self);
 				result = SQL_ERROR;
 				break;
@@ -775,8 +789,7 @@ QueryInfo qi;
 		mylog("   about to begin a transaction on statement = %u\n", self);
 		res = CC_send_query(conn, "BEGIN", NULL);
 		if (QR_aborted(res)) {
-			self->errormsg = "Could not begin a transaction";
-			self->errornumber = STMT_EXEC_ERROR;
+			SC_set_error(self, STMT_EXEC_ERROR, "Could not begin a transaction");
 			SC_log_error(func, "", self);
 			return SQL_ERROR;
 		}
@@ -788,8 +801,7 @@ QueryInfo qi;
 		QR_Destructor(res);
 
 		if (!ok) {
-			self->errormsg = "Could not begin a transaction";
-			self->errornumber = STMT_EXEC_ERROR;
+			SC_set_error(self, STMT_EXEC_ERROR, "Could not begin a transaction");
 			SC_log_error(func, "", self);
 			return SQL_ERROR;
 		}
@@ -873,9 +885,9 @@ QueryInfo qi;
 		was_nonfatal = QR_command_nonfatal(self->result);
 
 		if ( was_ok)
-			self->errornumber = STMT_OK;
+			SC_set_errornumber(self, STMT_OK);
 		else
-			self->errornumber = was_nonfatal ? STMT_INFO_ONLY : STMT_ERROR_TAKEN_FROM_BACKEND;
+			SC_set_errornumber(self, was_nonfatal ? STMT_INFO_ONLY : STMT_ERROR_TAKEN_FROM_BACKEND);
 
 		self->currTuple = -1; /* set cursor before the first tuple in the list */
 		self->current_col = -1;
@@ -888,8 +900,7 @@ QueryInfo qi;
 		if (numcols > 0) {
 			extend_bindings(self, numcols);
 			if (self->bindings == NULL) {
-				self->errornumber = STMT_NO_MEMORY_ERROR;
-				self->errormsg = "Could not get enough free memory to store the binding information";
+				SC_set_error(self, STMT_NO_MEMORY_ERROR, "Could not get enough free memory to store the binding information");
 				SC_log_error(func, "", self);
 				return SQL_ERROR;
 			}
@@ -900,8 +911,7 @@ QueryInfo qi;
 	} else {		/* Bad Error -- The error message will be in the Connection */
 
 		if (self->statement_type == STMT_TYPE_CREATE) {
-			self->errornumber = STMT_CREATE_TABLE_ERROR;
-			self->errormsg = "Error creating the table";
+			SC_set_error(self, STMT_CREATE_TABLE_ERROR, "Error creating the table");
 			/*	This would allow the table to already exists, thus appending
 				rows to it.  BUT, if the table didn't have the same attributes,
 				it would fail.
@@ -909,23 +919,22 @@ QueryInfo qi;
 			*/
 		}
 		else {
-			self->errornumber = STMT_EXEC_ERROR;
-			self->errormsg = "Error while executing the query";
+			SC_set_error(self, STMT_EXEC_ERROR, "Error while executing the query");
 		}
 
 		if ( ! self->internal)
 			CC_abort(conn);
 	}
 
-	if (self->errornumber == STMT_OK)
+	if (SC_get_errornumber(self) == STMT_OK)
 		return SQL_SUCCESS;
 
 	else {
 		/* Modified, 2000-04-29, Zoltan */
-		if (self->errornumber == STMT_INFO_ONLY)
-		    self->errormsg = "Error while executing the query (non-fatal)";
+		if (SC_get_errornumber(self) == STMT_INFO_ONLY)
+		    SC_set_errormsg(self, "Error while executing the query (non-fatal)");
 		else
-		    self->errormsg = "Unknown error";
+		    SC_set_errormsg(self, "Unknown error");
 		SC_log_error(func, "", self);
 		return SQL_ERROR;
 	}
@@ -938,8 +947,8 @@ SC_log_error(char *func, char *desc, StatementClass *self)
 #define nullcheck(a) (a ? a : "(NULL)")
 #endif
 	if (self) {
-		qlog("STATEMENT ERROR: func=%s, desc='%s', errnum=%d, errmsg='%s'\n", func, desc, self->errornumber, nullcheck(self->errormsg));
-		mylog("STATEMENT ERROR: func=%s, desc='%s', errnum=%d, errmsg='%s'\n", func, desc, self->errornumber, nullcheck(self->errormsg));
+		qlog("STATEMENT ERROR: func=%s, desc='%s', errnum=%d, errmsg='%s'\n", func, desc, self->__error_number, nullcheck(self->__error_message));
+		mylog("STATEMENT ERROR: func=%s, desc='%s', errnum=%d, errmsg='%s'\n", func, desc, self->__error_number, nullcheck(self->__error_message));
 		qlog("                 ------------------------------------------------------------\n");
 		qlog("                 hdbc=%u, stmt=%u, result=%u\n", self->hdbc, self, self->result);
 		qlog("                 manual_result=%d, prepare=%d, internal=%d\n", self->manual_result, self->prepare, self->internal);
