@@ -260,7 +260,6 @@ ConnectionClass *rv;
 		rv->DataSourceToDriver = NULL;
 		rv->DriverToDataSource = NULL;
 
-
 		/*	Initialize statement options to defaults */
 		/*	Statements under this conn will inherit these options */
 
@@ -864,7 +863,8 @@ QResultClass *
 CC_send_query(ConnectionClass *self, char *query, QueryInfo *qi)
 {
 QResultClass *result_in, *res = NULL;
-char id, swallow;
+int id;
+char swallow;
 SocketClass *sock = self->sock;
 static char msgbuffer[MAX_MESSAGE_LEN+1];
 char cmdbuffer[MAX_MESSAGE_LEN+1];	/*// QR_set_command() dups this string so dont need static */
