@@ -296,6 +296,20 @@ on_button61_clicked                    (GtkButton       *button,
 	gtk_widget_destroy( GTK_WIDGET( user_data ));
 }
 
+void
+on_dialog1_close                       (GtkDialog       *dialog,
+		                         gpointer         user_data)
+{ 
+	printf( "close\n" );
+} 
+  
+void
+on_dialog1_destroy                     (GtkObject       *object,
+		                         gpointer         user_data)
+{                   
+	        gtk_exit (0);
+	printf( "exit\n" );
+}                   
 
 void
 on_clist6_realize                      (GtkWidget       *widget,
@@ -316,7 +330,7 @@ on_clist6_realize                      (GtkWidget       *widget,
 
 	sprintf( szINI, "%s/odbcinst.ini", odbcinst_system_file_path());
 
-	if ( iniOpen( &hIni, szINI, '#', '[', ']', '=', TRUE ) != INI_ERROR )
+	if ( iniOpen( &hIni, szINI, "#;", '[', ']', '=', TRUE ) != INI_ERROR )
 	{
 		gchar *array[ 4 ];
 
@@ -771,7 +785,7 @@ on_add_dsn_ok_clicked                    (GtkButton       *button,
 	
 					case ODBCINST_PROMPTTYPE_COMBOBOX:
 					case ODBCINST_PROMPTTYPE_FILENAME:
-					default: /* PROMPTTYPE_TEXTEDIT */
+					default: /* PROMPTTYPE_TEXTEDIT AND PROMPTTYPE_TEXTEDIT_PASSWORD */
 					{
 						GtkWidget *entry;
 						gchar *ptr;
@@ -1081,7 +1095,7 @@ add_dsn                    				(GtkButton       *button,
 						}
 						break;
 
-						default: /* PROMPTTYPE_TEXTEDIT */
+						default: /* PROMPTTYPE_TEXTEDIT and PROMPTTYPE_TEXTEDIT_PASSWORD */
 						{
   							GtkWidget *entry;
 
@@ -1094,6 +1108,9 @@ add_dsn                    				(GtkButton       *button,
 
 							gtk_entry_set_text( GTK_ENTRY( entry ), hCurProperty->szValue );
 
+						if (hCurProperty->nPromptType == ODBCINST_PROMPTTYPE_TEXTEDIT_PASSWORD) 
+                                                        gtk_entry_set_visibility(GTK_ENTRY(entry),false);
+
   							gtk_widget_show (entry);
   							gtk_table_attach (GTK_TABLE (table8), entry, 1, 2, 
 											slot, slot + 1,
@@ -1471,7 +1488,7 @@ edit_dsn                   				(GtkButton       *button,
 						}
 						break;
 
-						default: /* PROMPTTYPE_TEXTEDIT */
+						default: /* PROMPTTYPE_TEXTEDIT and  PROMPTTYPE_TEXTEDIT_PASSWORD */
 						{
   							GtkWidget *entry;
 
@@ -1484,6 +1501,9 @@ edit_dsn                   				(GtkButton       *button,
 
 							gtk_entry_set_text( GTK_ENTRY( entry ), value );
 
+						if (hCurProperty->nPromptType == ODBCINST_PROMPTTYPE_TEXTEDIT_PASSWORD) 
+                                                        gtk_entry_set_visibility(GTK_ENTRY(entry),false);
+
   							gtk_widget_show (entry);
   							gtk_table_attach (GTK_TABLE (table8), entry, 1, 2, 
 											slot, slot + 1,
