@@ -39,7 +39,7 @@ int iniPropertyValue( char *pszString, char *pszProperty, char *pszValue, char c
 				if ( pValue )
 				{
 					/* truncate any other data */
-					pValueLastChar = (char *)strchr( pValue, (int)szPropertySep );
+					pValueLastChar = (char *)strchr( pValue, szPropertySep[ 0 ] );
 					if ( pValueLastChar )
 						pValueLastChar[0] = '\0';
 
