@@ -311,14 +311,18 @@ memcpy (dest, src, size)
      const lt_ptr src;
      size_t size;
 {
-  size_t i = 0;
-
-  for (i = 0; i < size; ++i)
+    size_t i = 0;
+    char *destptr;
+    char *srcptr;
+ 
+    destptr = (char*) dest;
+    srcptr  = (char*) src;
+ 
+    for (i = 0; i < size; ++i)
     {
-      dest[i] = src[i];
+        destptr[i] = srcptr[i];
     }
-
-  return dest;
+    return dest;
 }
 
 #  endif /* !HAVE_BCOPY */
@@ -957,6 +961,12 @@ lt_estrdup (str)
 #  define DLERROR(arg)	LT_DLSTRERROR (arg)
 #endif
 
+#ifdef RTLD_MEMBER
+# define LT_MEMBER  RTLD_MEMBER
+#else
+# define LT_MEMBER  0
+#endif
+
 static lt_module
 sys_dl_open (loader_data, filename)
      lt_user_data loader_data;
@@ -970,10 +980,14 @@ sys_dl_open (loader_data, filename)
 #ifdef QNX_LIBLTDL
   lt_module   module   = dlopen (filename, LT_GLOBAL | LT_LAZY_OR_NOW);
 #else
-#ifdef LT_GROUP
-  lt_module   module   = dlopen (filename, LT_GROUP | LT_LAZY_OR_NOW);
+#ifdef WITHOUT_RTLD_GROUP
+  lt_module   module   = dlopen (filename, LT_LAZY_OR_NOW | LT_MEMBER );
 #else
-  lt_module   module   = dlopen (filename, LT_LAZY_OR_NOW);
+#ifdef LT_GROUP
+  lt_module   module   = dlopen (filename, LT_GROUP | LT_LAZY_OR_NOW | LT_MEMBER );
+#else
+  lt_module   module   = dlopen (filename, LT_LAZY_OR_NOW | LT_MEMBER );
+#endif
 #endif
 #endif
 
@@ -2273,7 +2287,7 @@ find_handle_callback (filename, data, ignored)
   int		found	= access (filename, R_OK);
 
   /* Bail out if file cannot be read...  */
-  if (!found)
+  if (found)
     return 0;
 
   /* Try to dlopen the file, but do not continue searching in any
