@@ -43,12 +43,58 @@ int ODBCINSTConstructProperties( char *pszDriver, HODBCINSTPROPERTY *hFirstPrope
 #endif
 
 	/* GET DRIVER SETUP FILE NAME FOR GIVEN DRIVER */
-	if ( iniOpen( &hIni, szIniName, '#', '[', ']', '=', FALSE ) != INI_SUCCESS )
+#ifdef __OS2__
+	if ( iniOpen( &hIni, szIniName, "#;", '[', ']', '=', FALSE, 1L ) != INI_SUCCESS )
+#else
+	if ( iniOpen( &hIni, szIniName, "#;", '[', ']', '=', FALSE ) != INI_SUCCESS )
+#endif
 	{
 		inst_logPushMsg( __FILE__, __FILE__, __LINE__, LOG_CRITICAL, ODBC_ERROR_GENERAL_ERR, "Could not open odbcinst.ini" );
 		return ODBCINST_ERROR;
 	}
 
+#ifdef PLATFORM64
+	/* ASSUME USER FRIENDLY NAME FOR STARTERS */
+	if ( iniPropertySeek( hIni, pszDriver, "Setup64", "" ) == INI_SUCCESS )
+	{
+	}
+	else if ( iniPropertySeek( hIni, pszDriver, "Setup", "" ) == INI_SUCCESS )
+	{
+	}
+	else
+	{
+		/* NOT USER FRIENDLY NAME I GUESS SO ASSUME DRIVER FILE NAME */
+		if ( iniPropertySeek( hIni, "", "Driver64", pszDriver ) == INI_SUCCESS )
+		{
+			iniObject( hIni, szSectionName );
+			if ( iniPropertySeek( hIni, szSectionName, "Setup64", "" ) != INI_SUCCESS )
+			{
+				sprintf( szError, "Could not find Setup property for (%s) in system information", pszDriver );
+				inst_logPushMsg( __FILE__, __FILE__, __LINE__, LOG_CRITICAL, ODBC_ERROR_GENERAL_ERR, szError );
+				iniClose( hIni );
+				return ODBCINST_ERROR;
+			}
+		}
+		else if ( iniPropertySeek( hIni, "", "Driver", pszDriver ) == INI_SUCCESS )
+		{
+			iniObject( hIni, szSectionName );
+			if ( iniPropertySeek( hIni, szSectionName, "Setup", "" ) != INI_SUCCESS )
+			{
+				sprintf( szError, "Could not find Setup property for (%s) in system information", pszDriver );
+				inst_logPushMsg( __FILE__, __FILE__, __LINE__, LOG_CRITICAL, ODBC_ERROR_GENERAL_ERR, szError );
+				iniClose( hIni );
+				return ODBCINST_ERROR;
+			}
+		}
+		else
+		{
+			sprintf( szError, "Could not find driver (%s) in system information", pszDriver );
+			inst_logPushMsg( __FILE__, __FILE__, __LINE__, LOG_CRITICAL, ODBC_ERROR_GENERAL_ERR, szError );
+			iniClose( hIni );
+			return ODBCINST_ERROR;
+		}
+	}
+#else
 	/* ASSUME USER FRIENDLY NAME FOR STARTERS */
 	if ( iniPropertySeek( hIni, pszDriver, "Setup", "" ) != INI_SUCCESS )
 	{
@@ -72,6 +118,7 @@ int ODBCINSTConstructProperties( char *pszDriver, HODBCINSTPROPERTY *hFirstPrope
 			}
 		}
 	}
+#endif
 
 	iniValue( hIni, szDriverSetup );
 	iniClose( hIni );
