@@ -51,6 +51,7 @@ BOOL SQLManageDataSources( HWND hWnd )
      */
     if ( strncasecmp( hODBCINSTWnd->szGUI, "QT", 2 ) == 0 )
     {
+        char *p;
 /* 
  * This works and extension 'la' is probably more portable but we lose 
  * control of which version of the lib we load (it hard coded in the *.la)
@@ -68,22 +69,40 @@ BOOL SQLManageDataSources( HWND hWnd )
  *        sprintf( szGUILibFile, "%s/libodbcinstQ.so", DEFLIB_PATH );
  */
 
-/*
- * we need to find the extension to use as well
- */
-#ifdef SHLIBEXT
-        if ( strlen( SHLIBEXT ) > 0 )
-            sprintf( szGUILibFile, "libodbcinstQ%s", SHLIBEXT );
+        /*
+         * first look in the environment
+         */
+
+        p  = getenv( "ODBCINSTQ" );
+        if ( p )
+        {
+            strcpy( szGUILibFile, p );
+        }
         else
-            sprintf( szGUILibFile, "libodbcinstQ.so" );
+        {
+            SQLGetPrivateProfileString( "ODBC", "ODBCINSTQ", "", szGUILibFile, sizeof( szGUILibFile ), "odbcinst.ini" );
+
+            if ( strlen( szGUILibFile ) == 0 )
+            {
+                /*
+                 * we need to find the extension to use as well
+                 */
+
+#ifdef SHLIBEXT
+                if ( strlen( SHLIBEXT ) > 0 )
+                    sprintf( szGUILibFile, "libodbcinstQ%s.1", SHLIBEXT );
+                else
+                    sprintf( szGUILibFile, "libodbcinstQ.so.1" );
 #else
-        sprintf( szGUILibFile, "libodbcinstQ.so" );
+                sprintf( szGUILibFile, "libodbcinstQ.so.1" );
 #endif        
 
-        if ( lt_dladdsearchdir( DEFLIB_PATH ) )
-        {
-            inst_logPushMsg( __FILE__, __FILE__, __LINE__, LOG_CRITICAL, 
-                    ODBC_ERROR_GENERAL_ERR, (char*)lt_dlerror() );
+                if ( lt_dladdsearchdir( DEFLIB_PATH ) )
+                {
+                    inst_logPushMsg( __FILE__, __FILE__, __LINE__, LOG_CRITICAL, 
+                            ODBC_ERROR_GENERAL_ERR, (char*)lt_dlerror() );
+                }
+            }
         }
     }
     else
@@ -107,8 +126,6 @@ BOOL SQLManageDataSources( HWND hWnd )
 			nReturn = pSQLManageDataSources( (HWND)hODBCINSTWnd->hWnd );
 		else
 			inst_logPushMsg( __FILE__, __FILE__, __LINE__, LOG_CRITICAL, ODBC_ERROR_GENERAL_ERR, (char*)lt_dlerror() );
-
-		lt_dlclose( hDLL );
 	}
 	else
     {
@@ -126,8 +143,6 @@ BOOL SQLManageDataSources( HWND hWnd )
                 nReturn = pSQLManageDataSources( (HWND)hODBCINSTWnd->hWnd );
             else
                 inst_logPushMsg( __FILE__, __FILE__, __LINE__, LOG_CRITICAL, ODBC_ERROR_GENERAL_ERR, (char*)lt_dlerror() );
-
-            lt_dlclose( hDLL );
         }
 
 		inst_logPushMsg( __FILE__, __FILE__, __LINE__, LOG_CRITICAL, ODBC_ERROR_GENERAL_ERR, (char*)lt_dlerror() );
@@ -135,8 +150,3 @@ BOOL SQLManageDataSources( HWND hWnd )
 
 	return nReturn;
 }
-
-
-
-
-
