@@ -24,14 +24,14 @@
 #include "CCredits.h"
 
 
-class CAbout : public QWidget
+class CAboutDiagram : public QWidget
 {
     Q_OBJECT
 
 public:
 
-    CAbout( QWidget* parent = NULL, const char* name = NULL );
-    virtual ~CAbout();
+    CAboutDiagram( QWidget* pwidgetParent = NULL, const char* pszName = NULL );
+    virtual ~CAboutDiagram();
 
 protected slots:
     void pbODBCConfig_Clicked();
@@ -40,14 +40,20 @@ protected slots:
     void pbDriverManager_Clicked();
     void pbDriver_Clicked();
     void pbODBCDrivers_Clicked();
-    void pbCredits_Clicked();
     void pbApplication_Clicked();
-
-protected:
-	QFrame* qtarch_Frame_2;
-	QLabel* qtarch_Label_1;
-
-	void resizeEvent( QResizeEvent *p );
-
 };
+
+class CAbout : public QWidget
+{
+    Q_OBJECT
+
+public:
+
+    CAbout( QWidget* pwidgetPrent = NULL, const char* pszName = NULL );
+    virtual ~CAbout();
+
+protected slots:
+    void pbCredits_Clicked();
+};
+
 #endif 
