@@ -21,9 +21,13 @@
  *
  **********************************************************************
  *
- * $Id: attr.cpp,v 1.3 2001/12/20 17:26:26 lurcher Exp $
+ * $Id: attr.cpp,v 1.4 2003/12/01 16:37:17 lurcher Exp $
  *
  * $Log: attr.cpp,v $
+ * Revision 1.4  2003/12/01 16:37:17  lurcher
+ *
+ * Fix a bug in SQLWritePrivateProfileString
+ *
  * Revision 1.3  2001/12/20 17:26:26  lurcher
  *
  * More warnings removed
@@ -289,11 +293,17 @@ static attr_options stmt_options[] =
 	},
 	{ "SQL_ATTR_SIMULATE_CURSOR", SQL_ATTR_SIMULATE_CURSOR, 
 		{
+			{ "SQL_SC_NON_UNIQUE", SQL_SC_NON_UNIQUE }, 
+			{ "SQL_SC_TRY_UNIQUE", SQL_SC_TRY_UNIQUE }, 
+			{ "SQL_SC_UNIQUE", SQL_SC_UNIQUE }, 
 			{ NULL }
 		}, "2.0", SQL_INTEGER
 	},
 	{ "SQL_ATTR_USE_BOOKMARKS", SQL_ATTR_USE_BOOKMARKS,
 		{
+			{ "SQL_UB_ON", SQL_UB_ON }, 
+			{ "SQL_UB_OFF", SQL_UB_OFF }, 
+			{ "SQL_UB_VARIABLE", SQL_UB_VARIABLE }, 
 			{ NULL }
 		}, "2.0", SQL_INTEGER
 	},
@@ -386,6 +396,7 @@ static attr_options stmt_opt_options[] =
 		{
 			{ "SQL_UB_ON", SQL_UB_ON }, 
 			{ "SQL_UB_OFF", SQL_UB_OFF }, 
+			{ "SQL_UB_VARIABLE", SQL_UB_VARIABLE }, 
 			{ NULL }
 		}, "2.0", SQL_INTEGER
 	},
@@ -2254,8 +2265,8 @@ void dGetConnAttr::Ok()
 
 	attribute = conn_options[ types -> currentItem() ].attr;
 	txt.sprintf( "    Attribute: %s=%d", 
-		env_options[ types -> currentItem() ].text,
-		env_options[ types -> currentItem() ].attr );
+		conn_options[ types -> currentItem() ].text,
+		conn_options[ types -> currentItem() ].attr );
 	odbctest -> out_win -> insertLineLimited( txt );
 
 	b_len = atoi( buffer_len -> text().ascii());
@@ -2337,7 +2348,7 @@ void dGetConnAttr::Ok()
 	if ( SQL_SUCCEEDED( ret ))
 	{
 		int index = types -> currentItem();
-		switch( env_options[ index ].data_type )
+		switch( conn_options[ index ].data_type )
 		{
 			case SQL_CHAR:
 				txt.sprintf( "    *InfoValuePtr = \"%s\"", buf );
