@@ -21,9 +21,24 @@
  *
  **********************************************************************
  *
- * $Id: tools.cpp,v 1.3 2002/01/21 18:00:51 lurcher Exp $
+ * $Id: tools.cpp,v 1.6 2003/12/01 16:37:17 lurcher Exp $
  *
  * $Log: tools.cpp,v $
+ * Revision 1.6  2003/12/01 16:37:17  lurcher
+ *
+ * Fix a bug in SQLWritePrivateProfileString
+ *
+ * Revision 1.5  2003/11/13 15:12:54  lurcher
+ *
+ * small change to ODBCConfig to have the password field in the driver
+ * properties hide the password
+ * Make both # and ; comments in ini files
+ *
+ * Revision 1.4  2003/09/29 08:56:10  lurcher
+ *
+ * Fix warning dialogs
+ * Check for headers before using them
+ *
  * Revision 1.3  2002/01/21 18:00:51  lurcher
  *
  * Assorted fixed and changes, mainly UNICODE/bug fixes
@@ -264,95 +279,74 @@ static int show_message_box( int style, const char *title, const char *msg )
     button1 = QMessageBox::NoButton;
     button2 = QMessageBox::NoButton;
 
-    if ( style & MB_ABORTRETRYIGNORE )
+    switch ( style & (MB_OK | MB_ABORTRETRYIGNORE | MB_OKCANCEL | MB_RETRYCANCEL | MB_YESNO | MB_YESNOCANCEL) )
     {
+      case MB_ABORTRETRYIGNORE:
         button0 = QMessageBox::Abort;
         button1 = QMessageBox::Retry;
         button2 = QMessageBox::Ignore;
-    }
-    else if ( style & MB_OKCANCEL )
-    {
+        break;
+      case MB_OKCANCEL:
         button0 = QMessageBox::Ok;
         button1 = QMessageBox::Cancel;
-    }
-    else if ( style & MB_RETRYCANCEL )
-    {
+        break;
+      case MB_RETRYCANCEL:
         button0 = QMessageBox::Retry;
         button1 = QMessageBox::Cancel;
-    }
-    else if ( style & MB_YESNO )
-    {
+        break;
+      case MB_YESNO:
         button0 = QMessageBox::Yes;
-        button0 = QMessageBox::No;
-    }
-    else if ( style & MB_YESNOCANCEL )
-    {
+        button1 = QMessageBox::No;
+        break;
+      case MB_YESNOCANCEL:
         button0 = QMessageBox::Yes;
-        button0 = QMessageBox::No;
-        button1 = QMessageBox::Cancel;
-    }
-    else
-    {
+        button1 = QMessageBox::No;
+        button2 = QMessageBox::Cancel;
+        break;
+      default:
         button0 = QMessageBox::Ok;
+        break;
     }
 
-    if ( style & MB_DEFBUTTON2 )
-    {
-        button1 |= QMessageBox::Default;
-    }
-    else if ( style & MB_DEFBUTTON3 )
-    {
-        button2 |= QMessageBox::Default;
-    }
-    else
+    switch ( style & (MB_DEFBUTTON1 | MB_DEFBUTTON2 | MB_DEFBUTTON3 ) ) 
     {
+      case MB_DEFBUTTON1:
+      default:
         button0 |= QMessageBox::Default;
+        break;
+      case MB_DEFBUTTON2:
+        button1 |= QMessageBox::Default;
+        break;
+      case MB_DEFBUTTON3:
+        button2 |= QMessageBox::Default;
+        break;
     }
 
-    if ( style & MB_ICONINFORMATION )
+    QMessageBox::Icon icon = QMessageBox::NoIcon ;
+    switch ( style & (MB_ICONINFORMATION | MB_ICONQUESTION | MB_ICONSTOP ) ) 
     {
-        QMessageBox mb( title,
-                msg,
-                QMessageBox::Information,
-                button0,
-                button1,
-                button2 );
-
-        return mb.exec();
+      case MB_ICONINFORMATION :
+        icon = QMessageBox::Information;
+        break;
+      case MB_ICONQUESTION:
+        icon = QMessageBox::Warning;
+        break;
+      case MB_ICONSTOP:
+        icon = QMessageBox::Critical;
+        break;
+      default:
+        icon = QMessageBox::NoIcon;
+        break;
     }
-    else if ( style & MB_ICONQUESTION )
-    {
-        QMessageBox mb( title,
-                msg,
-                QMessageBox::Warning,
-                button0,
-                button1,
-                button2 );
 
-        return mb.exec();
-    }
-    else if ( style & MB_ICONSTOP )
-    {
-        QMessageBox mb( title,
-                msg,
-                QMessageBox::Critical,
-                button0,
-                button1,
-                button2 );
+    QMessageBox mb( title,
+                    msg,
+                    icon,
+                    button0,
+                    button1,
+                    button2 );
 
-        return mb.exec();
-    }
-    else
-    {
-        QMessageBox mb( title,
-                msg,
-                QMessageBox::NoIcon,
-                button0,
-                button1,
-                button2 );
-
-        return mb.exec();
-    }
+    return mb.exec();
 }
 
 extern "C" {
@@ -425,7 +419,7 @@ void replace_ini_list( OdbcTest *parent )
     char szINI[ODBC_FILENAME_MAX+1];
 
     sprintf( szINI, "%s/%s", SYSTEM_FILE_PATH, "Gator.ini" ); 
-	if ( iniOpen( &hIni, szINI, '#', '[', ']', '=', TRUE ) != INI_ERROR )
+	if ( iniOpen( &hIni, szINI, "#;", '[', ']', '=', TRUE ) != INI_ERROR )
 	{
         for ( section *ent = ini_list.first(); ent != 0; ent = ini_list.next())
 
@@ -461,7 +455,7 @@ void remove_section( OdbcTest *parent, section *section )
     char szINI[ODBC_FILENAME_MAX+1];
 
     sprintf( szINI, "%s/%s", SYSTEM_FILE_PATH, "Gator.ini" ); 
-	if ( iniOpen( &hIni, szINI, '#', '[', ']', '=', TRUE ) != INI_ERROR )
+	if ( iniOpen( &hIni, szINI, "#;", '[', ']', '=', TRUE ) != INI_ERROR )
 	{
         iniObjectSeekSure( hIni, (char*)section -> get_section());
         iniObjectDelete( hIni );
@@ -491,7 +485,7 @@ void init_ini_list( OdbcTest *parent )
     ini_list.clear();
 
     sprintf( szINI, "%s/%s", SYSTEM_FILE_PATH, "Gator.ini" ); 
-	if ( iniOpen( &hIni, szINI, '#', '[', ']', '=', TRUE ) != INI_ERROR )
+	if ( iniOpen( &hIni, szINI, "#;", '[', ']', '=', TRUE ) != INI_ERROR )
 	{
 		iniObjectFirst( hIni );
 		while ( iniObjectEOL( hIni ) == FALSE )
@@ -1166,6 +1160,7 @@ void dManageAutoTest::Add()
                     test_list -> insertItem( p -> name());
                 }
             }
+            test_list -> setCurrentItem( 0 );
 
             ListSelect( test_name );
         }
@@ -1213,7 +1208,11 @@ void dManageAutoTest::ListSelect( const QString &name )
 
 void dManageAutoTest::Remove()
 {
-    QString autotest = test_list->text(test_list->currentItem());
+  int index = test_list -> currentItem();
+
+  if ( index >= 0 )
+  {
+    QString autotest = test_list->text(index);
 
     //
     // remove it from the Auto Test section
@@ -1276,6 +1275,8 @@ void dManageAutoTest::Remove()
             }
         }
     }
+    test_list -> setCurrentItem( 0 );
+  }
 }
 
 dManageAutoTest::dManageAutoTest( OdbcTest *parent, QString name )
@@ -1728,7 +1729,7 @@ dManageTestGroup::~dManageTestGroup()
 
 MYQListViewItem::MYQListViewItem( QListView * parent, MYQListViewItem *after,
         QString label1, QString label2 )
-    :QListViewItem( parent, label1, label2 )
+    :QListViewItem( parent, after, label1, label2 )
 {
     test_index = 0;
     test_group = label1;
@@ -2104,7 +2105,7 @@ void dRunAutoTests::CursorChanged( int state )
     }
 }
 
-void dRunAutoTests::add_auto_test( const char * test_name, MYQListViewItem *top )
+void dRunAutoTests::add_auto_test( const char * test_name, MYQListViewItem *top, MYQListViewItem **item )
 {
     section *s = find_section( test_name );
 
@@ -2157,7 +2158,7 @@ void dRunAutoTests::add_auto_test( const char * test_name, MYQListViewItem *top
                         return;
                     }
 
-                    MYQListViewItem *item = new MYQListViewItem( top, test_name );  
+                    *item = new MYQListViewItem( top, *item, test_name );  
                     MYQListViewItem *after = NULL;
 
                     for ( int i = 1; i <= count; i ++ )
@@ -2167,13 +2168,13 @@ void dRunAutoTests::add_auto_test( const char * test_name, MYQListViewItem *top
                         {
                             if ( after )
                             {
-                                MYQListViewItem *test = new MYQListViewItem( item, after, 
+                                MYQListViewItem *test = new MYQListViewItem( *item, after, 
                                         func_name, test_desc, i, test_name );  
                                 after = test;
                             }
                             else
                             {
-                                MYQListViewItem *test = new MYQListViewItem( item, func_name, test_desc, 
+                                MYQListViewItem *test = new MYQListViewItem( *item, func_name, test_desc, 
                                         i, test_name );  
                                 after = test;
                             }
@@ -2330,6 +2331,27 @@ dRunAutoTests::dRunAutoTests( OdbcTest *parent, QString name )
 
 
     MYQListViewItem *top = NULL;  
+    MYQListViewItem *last_test = NULL;  
+
+    //
+    // fill auto tests
+    //
+
+    s = find_section( "Auto Tests" );
+
+    if ( s )
+    {
+        prop *p;
+
+        top = new MYQListViewItem( tests, top, "All" );  
+        
+        for ( p = s -> first(); 
+                p != 0; 
+                p = s -> next())
+        {
+            add_auto_test( p -> name(), top, &last_test );
+        }
+    }
 
     //
     // fill individual tests
@@ -2347,36 +2369,16 @@ dRunAutoTests::dRunAutoTests( OdbcTest *parent, QString name )
             if ( s1 )
             {
                 top = new MYQListViewItem( tests, top, p -> name());  
-
+                last_test = NULL ;
                 for ( p1 = s1-> first(); 
                         p1 != 0; 
                         p1 = s1-> next())
                 {
-                    add_auto_test( p1 -> name(), top );
+                    add_auto_test( p1 -> name(), top, &last_test );
                 }
             }
         }
     }
-
-    //
-    // fill auto tests
-    //
-
-    s = find_section( "Auto Tests" );
-
-    if ( s )
-    {
-        prop *p;
-
-        top = new MYQListViewItem( tests, top, "All" );  
-
-        for ( p = s -> first(); 
-                p != 0; 
-                p = s -> next())
-        {
-            add_auto_test( p -> name(), top );
-        }
-    }
 }
 
 dRunAutoTests::~dRunAutoTests()
