@@ -31,7 +31,7 @@ HSQPCOMPARISON sqpStoreComparison( char *pszLValue, char *pszOperator, char *psz
     if (pszEscape)
         pComparison->cEscape    = pszEscape[1];
     else
-        pComparison->cEscape    = NULL;
+        pComparison->cEscape    = '\0';
 	
 #ifdef SQPDEBUG
 	printf( "[SQP][%s][%d] END: %s %s %s\n", __FILE__, __LINE__, pComparison->pszLValue, pComparison->pszOperator, pComparison->pszRValue );
